from visad.python.JPythonMethods import *
#
#This is the default editable user's jython library

import java
sys.add_package('visad');
sys.add_package('visad.python');

from visad.python.JPythonMethods import *
##########################################################################################################################################################
# some examples image calculations
##########################################################################################################################################################

##########################################################################################################################################################
# compute average of non-timesequence imagery
##########################################################################################################################################################
# Purpose: compute average value of all pixels in one image
# Creator: V. Venus
# 
import sys;
sys.add_package('org.apache.commons.math.stat.descriptive.moment');
from org.apache.commons.math.stat.descriptive.moment import Mean
from jarray import zeros
from java.lang import Float

def averageImages (a,b):
  lineValuesA = a.getFloats()
  lineValuesB = b.getFloats()
  values = zeros(2,"d")
  for i in xrange(a.getDomainSet().getLength()):
    # first fill the array 'values' with values
    values[0] = lineValuesA[0][i]
    values[1] = lineValuesB[0][i]
    #average = (lineValuesA[0][i] + lineValuesB[0][i])/2
    mean = Mean()
    average = mean.evaluate(values)
    lineValuesA[0][i] = average
  newd = a.clone()
  newd.setSamples(lineValuesA)
  return newd


##########################################################################################################################################################
# aggregate timesequence (call this jython function for image/grid sequences)
##########################################################################################################################################################
# Purpose: aggregate (per pixel) a timesequence
# Creator: V. Venus
# 


def aggregateSeq (timeSequence):
  import sys;
  sys.add_package('visad.meteorology');
  sys.add_package('GridUtil');
  # here we may choose various aggregator offered by the apache.org "math" package
  #sys.add_package('org.apache.commons.math.stat.descriptive.moment');
  #from org.apache.commons.math.stat.descriptive.moment import Mean
  sys.add_package('org.apache.commons.math.stat.descriptive.rank');
  from org.apache.commons.math.stat.descriptive.rank import Percentile

  from jarray import zeros
  from java.lang import Float
  from java.lang import Double

  singleBand = timeSequence.getSample(0)
  domain = singleBand.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()

  # Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  # Clone the incoming objects
  news = singleBand.clone()

  lineValuesA = singleBand.getFloats(0)

  alg = Percentile() #Mean()

  #print "Starting for loop ... number of iterations = ", len
  # calculate aggregates for all coordinates (crd) for all times (time)
  for crd in xrange(len):
      #print "Pixel nr. ", crd + 1, " of ", len 
      # get length of imageSequence
      seq = timeSequence.getDomainSet().getLength()
      # create array to store values 
      arr = zeros(seq, "d")
      # get all values for one coordinate for all times time and put them into arr
      #print "Entering nested forloop 1"
      for time in xrange(seq):
          # get the values
          # Pass in 0 as false so we don't copy the values
          #print "    Retrieving pixel info for image nr. ", time + 1, " of ", seq
          values = timeSequence.getSample(time).getFloats(0)
          #print "    Value for current pixel (#", crd + 1, "): ",values[0][crd]
          arr[time] = values[0][crd]
      #print "Exited nested forloop 1"
      # Calculate the aggregate
      #print "Starting aggregation..."
      aggregate = alg.evaluate(arr, Double(10))
      #print aggregate
      #print "Finished aggregation"
      # feed the aggregates back into the clone newd (singleBand/grid)
      lineValuesA[0][crd] = aggregate
  news.setSamples(lineValuesA)
  return news

##########################################################################################################################################################
# compute the old Cloud Index of timesequence imagery (call this jython function only for image sequences)
##########################################################################################################################################################
# Purpose: compute the Cloud Index of timesequence imagery (call this jython function only for image sequences)
# Creator: V. Venus
# 

def getCloudIndexOldHeliosatImageSeqN52(histTimeSequence,histCsaSeq,currTimeSequence,currCsaSeq,user_pGround,user_pCloud):
  import sys;
  sys.add_package('visad.meteorology');
  sys.add_package('GridUtil');

  sys.add_package('org.apache.commons.math.stat.descriptive.rank');
  from org.apache.commons.math.stat.descriptive.rank import Percentile

  sys.add_package('org.apache.commons.math.analysis');
  from org.apache.commons.math.analysis import UnivariateRealInterpolator, UnivariateRealFunction
  from org.apache.commons.math.analysis import SplineInterpolator, PolynomialSplineFunction, PolynomialFunction

  from java.util import TimeZone
  from jarray import zeros, array
  from java.lang import Float, Double, Object

  from visad import DateTime
  from visad.meteorology import ImageSequence
  from visad.meteorology import SingleBandedImage

  singleBand = currTimeSequence.getSample(0)
  domain = singleBand.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()

  # Clone the incoming objects
  news = singleBand.clone()
  newd = currTimeSequence.clone()

  alg = Percentile()
  pG = Double(user_pGround) # Sets the value of the quantile field (which determines what percentile is computed when 
                            # evaluate() is called with quantile argument).

  pC = Double(user_pCloud) # Sets the value of the quantile field (which determines what percentile is computed when 
                           # evaluate() is called with quantile argument). Suggested values are 98-th percentile


  binLength = 13
  knot4 = zeros(binLength,"d")
  bin4 = zeros(binLength,"d")
  bin = zeros(binLength, Object)
  count = zeros(1,"d")

  # get length of imageSequence
  seq = histTimeSequence.getDomainSet().getLength()
  currSeq = currTimeSequence.getDomainSet().getLength()
  # create array to store values 
  for i in xrange(binLength):
      bin[i] = zeros(seq, "d")

  for crd in xrange(len):
      count = zeros(binLength, "i")
      # get all temperatures for one coordinate for all times time and put them into arr
      #print "Entering nested forloop 1"
      for time in xrange(seq):
          lineValuesHistRefl = histTimeSequence.getSample(time).getFloats(0)
          refl = lineValuesHistRefl[0][crd]
          print "    Refl for current pixel (#", crd + 1, "): ",refl

          CA = histCsaSeq.getSample(time).getFloats(0)
          print "    CA for current pixel (#", crd + 1, "): ",CA[0][crd]

          if (refl > 0.75):
            if (CA[0][crd] < 10):
              #print "CA < 10"
              bin[0][count[0]] = refl
              #knot[0] = 0
              count[0] = count[0] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 10) and (CA[0][crd] < 20):
              #print "(CA >= 10) and (CA < 20)"
              bin[1][count[1]] = refl
              #knot[1] = 10
              count[1] = count[1] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 20) and (CA[0][crd] < 30):
              #print "(CA >= 20) and (CA < 30)"
              bin[2][count[2]] = refl
              #knot[2] = 20
              count[2] = count[2] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 30) and (CA[0][crd] < 40):
              #print "(CA >= 30) and (CA < 40)"
              bin[3][count[3]] = refl
              #knot[3] = 30
              count[3] = count[3] + 1
              #print "CA: ",CA," refl: ", refl[0][i]
            elif (CA[0][crd] >= 40) and (CA[0][crd] < 50):
              #print "(CA >= 40) and (CA < 50)"
              bin[4][count[4]] = refl
              #knot[4] = 40
              count[4] = count[4] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 50) and (CA[0][crd] < 60):
              #print "(CA >= 50) and (CA < 60)"
              bin[5][count[5]] = refl
              #knot[5] = 50
              count[5] = count[5] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 60) and (CA[0][crd] < 70):
              #print "(CA >= 60) and (CA < 70)"
              bin[6][count[6]] = refl
              #knot[6] = 60
              count[6] = count[6] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 70) and (CA[0][crd] < 80):
              #print "CA > 70"
              bin[7][count[7]] = refl
              #knot[7] = 70
              count[7] = count[7] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 80) and (CA[0][crd] < 90):
              #print "CA > 80"
              bin[8][count[8]] = refl
              #knot[8] = 80
              count[8] = count[8] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 90) and (CA[0][crd] < 100):
              #print "CA > 90"
              bin[9][count[9]] = refl
              #knot[9] = 90
              count[9] = count[9] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 100) and (CA[0][crd] < 110):
              #print "CA > 100"
              bin[10][count[10]] = refl
              #knot[10] = 100
              count[10] = count[10] + 1
              #print "CA: ",CA," refl: ", refl    
            elif (CA[0][crd] >= 110) and (CA[0][crd] < 120):
              #print "CA > 110"
              bin[11][count[11]] = refl
              #knot[11] = 110
              count[11] = count[11] + 1
              #print "CA: ",CA," refl: ", refl 
            elif (CA[0][crd] >= 120) and (CA[0][crd] < 130):
              #print "CA > 120"
              bin[12][count[12]] = refl
              #knot[12] = 120
              count[12] = count[12] + 1
              #print "CA: ",CA," refl: ", refl
      knot = 0
      valid = 0

      #print "binlength: ", binLength
      for i in xrange(binLength):
        if (count[i] > 0): 
          print "valid"
          binTemp = zeros(count[i], "d")
          for j in xrange (0,count[i]):
            binTemp[j] = bin[i][j]
          print "binTemp: ", binTemp
          bin4[valid] = alg.evaluate(binTemp, pG)
          knot4[valid] = knot
          valid = valid + 1
        else:
          print "empty bin at ", i
          #invalid = invalid + 1
        knot = knot + 10
      cnt = 0
      for i in xrange(5):
        if (count[i] > 0): 
          cnt = cnt + count[i]
      binGround = zeros(cnt, "d")
      #x = 0
      j = 0
      i = 0
      index = 0
      for i in xrange(5):
        if (count[i] > 0): 
          for j in xrange (0,count[i]):
            binGround[index] = bin[i][j]
            index = index + 1
        else:
          print "empty bin at ", i
      print "binGround: ", binGround
      bin1to5p = alg.evaluate(binGround, pG)
      print "bin1to5pth: ", bin1to5p

      bin4Temp = zeros(valid, "d")
      knot4Temp = zeros(valid, "d")
      for j in xrange (0,valid):
         bin4Temp[j] = bin4[j]
         knot4Temp[j] =  knot4[j]
      print "knot4: ", knot4Temp
      print "bin4: ", bin4Temp
      knotLength = valid
      print "knotLength: ", knotLength  
       
      if (knotLength > 2): # At least 3 datapoints are required to compute a spline interpolant
        minCSA = knot4Temp[0]
        maxCSA = knot4Temp[knotLength-1]
        interpolator = SplineInterpolator(); 
        function = interpolator.interpolate(knot4Temp,bin4Temp);
        for time in xrange(currSeq):
          valuesCSA = currCsaSeq.getSample(time).getFloats(0)

          #Reflectivity of ground(sg)
          CSA = valuesCSA[0][crd] 
          print "CSA: ", CSA, " knotlength: ", knotLength * 10
          if CSA > maxCSA:
            CSA = maxCSA
          if CSA < minCSA:
            CSA = minCSA
          print "CSA: ", CSA, " knotlength: ", knotLength * 10
          sg = function.value(Double(CSA))
          print "Ground refl for current pixel (#", crd + 1, ") for current time (#", time + 1, "out of ", currSeq, "): ", sg
          singleB = newd.getSample(time)
          lineValuesCurrRefl  = currTimeSequence.getSample(time).getFloats(0)
          values  = newd.getSample(time).getFloats(0)

          #Reflectivity of current pixel
          sc = lineValuesCurrRefl[0][crd]
          print "Refl for current pixel (#", crd + 1, ") for current time (#", time + 1, "out of ", currSeq, "): ", sc

          # Cloud index(n)  rho_cloud =0.81 (Dagestad, personal comm.) or use pth-percentile through user_pCloud
          #n = (lineValuesA[0][i] * 0.01  - 0.17) / (0.81-0.17) # 0.17 is from weather station data   
          n = (sc - sg) / (81 - sg)
          if n <= -0.2:
            print "n <= -0.2"
            K = 1.2
            print "n: ",n," K: ", K
          elif n > 1.1:
            print "n >= 1.1"
            K = 0.05
            print "n: ",n," K: ", K
          elif n > 0.80:
            print "n > 0.80"
            K = 2.0667 - 3.6667 * n + 1.6667 * n * n
            print "n: ",n," K: ", K
          else: 
            print "n <= 0.80"
            K = 1 - n
            print "n: ",n," K: ", K

          try:
              values[0][crd] = K
          except:
              values[0][crd] = Float.NaN #error, fill pixel value with "missing" or NaN

          #newA.setSample(time,getRadHeliosatN52image(seqA))
      else:
        values[0][crd] = Float.NaN; #  Sorry, at least 3 datapoints are required to compute a spline interpolant
  singleB.setSamples(values,0)
  newd.setSample(time,singleB)
  return newd

##########################################################################################################################################################
# compute the Cloud Index of timesequence imagery (call this jython function only for image sequences)
##########################################################################################################################################################
# Purpose: compute the Cloud Index of timesequence imagery (call this jython function only for image sequences)
# Creator: V. Venus
# 


def getCloudIndexHeliosatImageSeqN52(histTimeSequence,histCsaSeq,currTimeSequence,currCsaSeq,user_pGround,user_pCloud):
  import sys;
  sys.add_package('visad.meteorology');
  sys.add_package('GridUtil');

  sys.add_package('org.apache.commons.math.stat.descriptive.rank');
  from org.apache.commons.math.stat.descriptive.rank import Percentile

  sys.add_package('org.apache.commons.math.analysis');
  from org.apache.commons.math.analysis import UnivariateRealInterpolator, UnivariateRealFunction
  from org.apache.commons.math.analysis import SplineInterpolator, PolynomialSplineFunction, PolynomialFunction

  from java.util import TimeZone
  from jarray import zeros, array
  from java.lang import Float, Double, Object

  from visad import DateTime
  from visad.meteorology import ImageSequence
  from visad.meteorology import SingleBandedImage

  singleBand = currTimeSequence.getSample(0)
  domain = singleBand.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()

  # Clone the incoming objects
  news = singleBand.clone()
  newd = currTimeSequence.clone()

  alg = Percentile()
  pG = Double(user_pGround) # Sets the value of the quantile field (which determines what percentile is computed when 
                            # evaluate() is called with quantile argument). Suggested values are 4-th percentile

  pC = Double(user_pCloud) # Sets the value of the quantile field (which determines what percentile is computed when 
                           # evaluate() is called with quantile argument). Suggested values are 98-th percentile

  knot4 = zeros(13,"d")
  bin4 = zeros(13,"d")
  bin = zeros(13, Object)
  binLength = 13
  count = zeros(1,"d")

  # get length of imageSequence
  seq = histTimeSequence.getDomainSet().getLength()
  currSeq = currTimeSequence.getDomainSet().getLength()
  # create array to store ground shape values 
  for i in range(binLength):
      bin[i] = zeros(seq, "d")

  # create array to store cloud reflectance 
  arr = zeros(seq, "d")

  for crd in xrange(len):
      count = zeros(binLength, "i")
      # get all temperatures for one coordinate for all times time and put them into arr
      #print "Entering nested forloop 1"
      for time in xrange(seq):
          refl = histTimeSequence.getSample(time).getFloats(0)
          #print "    Value for current pixel (#", crd + 1, "): ",values[0][crd]
          CA = histCsaSeq.getSample(time).getFloats(0)
          refl = refl[0][crd]
          if (refl >  0.75):
            if (CA[0][crd] < 10):
              #print "CA < 10"
              bin[0][count[0]] = refl
              #knot[0] = 0
              count[0] = count[0] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 10) and (CA[0][crd] < 20):
              #print "(CA >= 10) and (CA < 20)"
              bin[1][count[1]] = refl
              #knot[1] = 10
              count[1] = count[1] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 20) and (CA[0][crd] < 30):
              #print "(CA >= 20) and (CA < 30)"
              bin[2][count[2]] = refl
              #knot[2] = 20
              count[2] = count[2] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 30) and (CA[0][crd] < 40):
              #print "(CA >= 30) and (CA < 40)"
              bin[3][count[3]] = refl
              #knot[3] = 30
              count[3] = count[3] + 1
              #print "CA: ",CA," refl: ", refl[0][i]
            elif (CA[0][crd] >= 40) and (CA[0][crd] < 50):
              #print "(CA >= 40) and (CA < 50)"
              bin[4][count[4]] = refl
              #knot[4] = 40
              count[4] = count[4] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 50) and (CA[0][crd] < 60):
              #print "(CA >= 50) and (CA < 60)"
              bin[5][count[5]] = refl
              #knot[5] = 50
              count[5] = count[5] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 60) and (CA[0][crd] < 70):
              #print "(CA >= 60) and (CA < 70)"
              bin[6][count[6]] = refl
              #knot[6] = 60
              count[6] = count[6] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 70) and (CA[0][crd] < 80):
              #print "CA > 70"
              bin[7][count[7]] = refl
              #knot[7] = 70
              count[7] = count[7] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 80) and (CA[0][crd] < 90):
              #print "CA > 80"
              bin[8][count[8]] = refl
              #knot[8] = 80
              count[8] = count[8] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 90) and (CA[0][crd] < 100):
              #print "CA > 90"
              bin[9][count[9]] = refl
              #knot[9] = 90
              count[9] = count[9] + 1
              #print "CA: ",CA," refl: ", refl
            elif (CA[0][crd] >= 100) and (CA[0][crd] < 110):
              #print "CA > 100"
              bin[10][count[10]] = refl
              #knot[10] = 100
              count[10] = count[10] + 1
              #print "CA: ",CA," refl: ", refl    
            elif (CA[0][crd] >= 110) and (CA[0][crd] < 120):
              #print "CA > 110"
              bin[11][count[11]] = refl
              #knot[11] = 110
              count[11] = count[11] + 1
              #print "CA: ",CA," refl: ", refl 
            elif (CA[0][crd] >= 120) and (CA[0][crd] < 130):
              #print "CA > 120"
              bin[12][count[12]] = refl
              #knot[12] = 120
              count[12] = count[12] + 1
              #print "CA: ",CA," refl: ", refl
      knot = 0
      valid = 0

      #print "binlength: ", binLength
      for i in xrange(binLength):
        if (count[i] > 0): 
          print "valid"
          binTemp = zeros(count[i], "d")
          for j in xrange (0,count[i]):
            binTemp[j] = bin[i][j]
          print "binTemp: ", binTemp
          bin4[valid] = alg.evaluate(binTemp, pG)
          knot4[valid] = knot
          valid = valid + 1
        else:
          print "empty bin at ", i
          #invalid = invalid + 1
        knot = knot + 10
      cnt = 0
      for i in xrange(5):
        if (count[i] > 0): 
          cnt = cnt + count[i]
      binCloud = zeros(cnt, "d")
      #x = 0
      j = 0
      i = 0
      index = 0
      for i in xrange(5):
        if (count[i] > 0): 
          for j in xrange (0,count[i]):
            binCloud[index] = bin[i][j]
            index = index + 1
        else:
          print "empty bin at ", i
      print "binCloud: ", binCloud
      rc = alg.evaluate(binCloud, pC)
      print "rc: ", rc

      bin4Temp = zeros(valid, "d")
      knot4Temp = zeros(valid, "d")
      for j in xrange (0,valid):
         bin4Temp[j] = bin4[j]
         knot4Temp[j] =  knot4[j]
      print "knot4: ", knot4Temp
      print "bin4: ", bin4Temp
      knotLength = valid
      print "knotLength: ", knotLength         
      if (knotLength > 2): # At least 3 datapoints are required to compute a spline interpolant
        interpolator = SplineInterpolator(); 
        function = interpolator.interpolate(knot4Temp,bin4Temp);
        for time in xrange(currSeq):
          valuesCSA = currCsaSeq.getSample(time).getFloats(0)

          #Reflectivity of ground(sg)
          CSA = valuesCSA[0][crd] 
          print "CSA: ", CSA, " knotlength: ", knotLength * 10
          if (CSA > (knotLength-1*10)):
            CSA = (knotLength-1)*10
          if CSA < 0:
            CSA = 0
          sg = function.value(Double(CSA))
          values  = currTimeSequence.getSample(time).getFloats(0)
          #singleB = currTimeSequence.getSample(time)

          #Reflectivity of current pixel
          sc = values[0][crd]
          print "Reflectivity of current pixel: ", sc

          # Cloud index(n)  rho_cloud =0.81 (Dagestad, personal comm.) or use pth-percentile through user_pCloud
          #n = (lineValuesA[0][i] * 0.01  - 0.17) / (0.81-0.17) # 0.17 is from weather station data   
          n = (sc - sg) / (rc - sg)
          if n <= -0.2:
            print "n <= -0.2"
            K = 1.2
            print "n: ",n," K: ", K
          elif n > 1.1:
            print "n >= 1.1"
            K = 0.05
            print "n: ",n," K: ", K
          elif n > 0.80:
            print "n > 0.80"
            K = 2.0667 - 3.6667 * n + 1.6667 * n * n
            print "n: ",n," K: ", K
          else: 
            print "n <= 0.80"
            K = 1 - n
            print "n: ",n," K: ", K

          #Correction of k#new Kch = Kch � 0.01(8 TST � 104)
          #Kc = K - 0.01 * (8 * utc - 104)
          #if Kc > 1.2:
          #  Kcr = 1.2
          #elif Kc < 0.05:
          #  Kcr = 0.05
          #else:
          #  Kcr = Kc

          try:
              values[0][crd] = K
          except:
              values[0][crd] = Float.NaN #error, fill pixel value with "missing" or NaN

          #newA.setSample(time,getRadHeliosatN52image(seqA))
          #singleB.setSamples(values,0)
          #newd.setSample(time,singleB)
      else:
        values[0][crd] = Float.NaN # sorry, all pixels are "missing" aka NaN: at least 3 datapoints are required to compute a spline interpolant
  return newd


def test(currTimeSequence):
 from java.lang import Float, Double, Object
 newd = currTimeSequence.clone()
 currSeq = currTimeSequence.getDomainSet().getLength()

 for time in xrange(currSeq):
       lineValues  = currTimeSequence.getSample(time).getFloats(0)
       singleB = currTimeSequence.getSample(time)
       for idx in range(len(lineValues[0])):
           value = lineValues[0][idx]
           #print value
           lineValues[0][idx] = value;
       #singleB.setSamples(lineValues,0)
       #newd.setSample(time,singleB)
 return newd

##########################################################################################################################################################
# compute pth-percentile of timesequence imagery (call this jython function only for image sequences)
##########################################################################################################################################################
# Purpose: compute pth-percentile (per pixel) of timesequence imagery
# Creator: V. Venus
# 


def percentileImagesSeq(timeSequence,user_percentile):
  import sys;
  sys.add_package('visad.meteorology');
  sys.add_package('GridUtil');
  sys.add_package('org.apache.commons.math.stat.descriptive.rank');
  from org.apache.commons.math.stat.descriptive.rank import Percentile

  from java.util import TimeZone
  from jarray import zeros
  from java.lang import Float
  from java.lang import Double

  from visad import DateTime
  from visad.meteorology import ImageSequence
  from visad.meteorology import SingleBandedImage

  #print "Function getTempDiurnalImage started ..."
  singleBand = timeSequence.getSample(0)
  domain = singleBand.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()

  # Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  # Clone the incoming objects
  news = singleBand.clone()

  lineValuesA = singleBand.getFloats()

  # Initialize temperature algorithm

  # Get starttime of image sequence
  #tz = TimeZone.getTimeZone("GMT");
  #if isinstance(timeSequence, ImageSequence):
  #    sbi = timeSequence.getImage(int(0))
  #    dateTime = sbi.getStartTime()
  #elif isinstance(timeSequence, SingleBandedImage):
  #    dateTime = timeSequence.getStartTime()
  #year = dateTime.formattedString("yyyy", tz)
  #month = dateTime.formattedString("MM", tz)
  #day = dateTime.formattedString("dd", tz)
  #hour = float(dateTime.formattedString("H", tz))
  #minutes = float(dateTime.formattedString("mm", tz))

  alg = Percentile()
  p = Double(user_percentile) #Sets the value of the quantile field (determines what percentile is computed when evaluate() is called with quantile argument).

  # get length of imageSequence
  seq = timeSequence.getDomainSet().getLength()

  #print "Starting for loop ... number of iterations = ", len
  # calculate temperatures for all coordinates (crd) for all times (time)
  for crd in xrange(len):
      #print "Pixel nr. ", crd + 1, " of ", len 

      # get all temperatures for one coordinate for all times time and put them into arr
      #print "Entering nested forloop 1"
      # create array to store values for each pixel a new 
      arr = zeros(seq, "d")
      for time in xrange(seq):
          # get the values
          # Pass in 0 as false so we don't copy the values
          #print "    Retrieving pixel info for image nr. ", time + 1, " of ", seq 
          values = timeSequence.getSample(time).getFloats()
          #print "    Value for current pixel (#", crd + 1, "): ",values[0][crd]
          arr[time] = values[0][crd]
      #print "Exited nested forloop 1"
      # Calculate the temperatures
      #print "Starting temperature averaging"
      value = alg.evaluate(arr, p)
      #print value
      #print "Finished temperature calculation"
      # feed the temperatures back into the clone newd
      lineValuesA[0][crd] = value
  news.setSamples(lineValuesA,0)
  return news

##########################################################################################################################################################
# Clear sky radiation using FluxNet based on a image object using N52 framework operations
##########################################################################################################################################################
#
# Purpose: compute Satellite zenith angle (VZA, degrees) based on a image object
# Creator: V. Venus, J. Hendrikse
# Usable in: Images, Data Probe
# Grouped under: User-satellite land surface obs
# see functional describtion of algorithm at:
# http://adde.itc.nl/IDV/operation/org/n52/operation.html

sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
sys.add_package('org.n52.operation.fluxnet');
from org.n52.math import TerrainTriangulation
from org.n52.math import SatelliteTriangulation
from org.n52.operation.fluxnet import FluxNetAlgorithm 
from jarray import zeros

def getClearSkyRadImageN52(tsurf,emissurf,surftype,albsurf,cldphase,cldre,cldwc,cldtau,cldtemp,cldfrac,zen,tauhaze,o3,h2ocol):
  domain = tsurf.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
#Note: we want a copy of the range values so we don't pass in 0 (false) here
  rangeValues = tsurf.getFloats()
#Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
#Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
#Clone the incoming data object
  newd = tsurf.clone()
# :
#  tsurf  = tsurf.getFloats()
#  emissurf = emissurf.getFloats()
#  surftype = surftype.getFloats()
#  albsurf = albsurf.getFloats()
#  cldphase = cldphase.getFloats()
#  cldre = cldre.getFloats()
#  cldwc = cldwc.getFloats()
#  cldtau = cldtau.getFloats()
#  cldtemp = cldtemp.getFloats()
#  cldfrac = cldfrac.getFloats()
#  zen = zen.getFloats()
#  tauhaze = tauhaze.getFloats()
#  o3 = o3.getFloats()
#  h2ocol = h2ocol.getFloats()
# 270.05 0.954 5. 0.397 1. 56.3 0.107 4.1 219.9 0.97 27.67 0.17 6.647 6306.54
  tsurf1  = 270.05
  emissurf = 0.954
  surftype = 5.
  albsurf = 0.397
  cldphase = 1.
  cldre = 56.3
  cldwc = 0.107
  cldtau = 4.1
  cldtemp = 219.9
  cldfrac = 0.97
  zen = 27.67
  tauhaze = 0.17
  o3 = 6.647
  h2ocol = 6306.54
  tertr = TerrainTriangulation(6378169, 295.488065897)
  sattr = SatelliteTriangulation(tertr, 0, 0, 3, 35785831)
  zenithalg = sattr.getZenithAngleAlgorithm()

  fluxnetalg = FluxNetAlgorithm("http://adde.itc.nl/IDV/16_10.net","http://adde.itc.nl/IDV/FluxnetAlgorithmConfiguration.xml")

  #params = zeros(14,"tsurf")
  for i in xrange(len):
    # fill the data object with latitude and longitude coordinates in degrees 
    #params[0] = tsurf[0][i]
    #params[1] = emissurf[0][i]
    #params[2] = surftype[0][i]
    #params[3] = albsurf[0][i]
    #params[4] = cldphase[0][i]
    #params[5] = cldre[0][i]
    #params[6] = cldwc[0][i]
    #params[7] = cldtau[0][i]
    #params[8] = cldtemp[0][i]
    #params[9] = cldfrac[0][i]
    #params[10] = zen[0][i]
    #params[11] = tauhaze[0][i]
    #params[12] = o3[0][i]
    #params[13] = h2ocol[0][i]
    params = [tsurf1,emissurf,surftype,albsurf,cldphase,cldre,cldwc,cldtau,cldtemp,cldfrac,zen,tauhaze,o3,h2ocol]
    value = fluxnetalg.calculate(params)
    rangeValues[0][i] = value[0]
  newd.setSamples(rangeValues)
  return newd

##########################################################################################################################################
# Co-scaterring angle
##########################################################################################################################################
# Initialize the SatelliteTriangulation: 
# sattr = SatelliteTriangulation(tertr, 0, nadirLongitude, nadirPixelSize, nadirAltitude)
# one has to provide the following information: 
# satellite position, resolution [km] and altitude [m], 
# for FY-2C http://www.sat-nd.com/geo/28451.html
# nadirPixelSize = in km
# nadirLongitude = in decimal degrees (104.5 in the case of FY-2C)
# nadirAltitude  = in meters from the earth's surface (roughly 35794000 in the case of FY-2C, 35785831 MSG-8) 
 
import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');

from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence 
from visad.meteorology import SingleBandedImage

sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation 
from org.n52.math import SatelliteTriangulation 
from org.n52.math import SunTriangulation 

from jarray import zeros 
from java.lang import Float

def getCoscaterringAngleImageN52(d,nadirPixelSize,nadirLongitude,nadirAltitude):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  #rangeValues = d.getFloats()
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  #Clone the incoming data object
  newd = d.clone()
  
  tz = TimeZone.getTimeZone("GMT");
  if isinstance(d, ImageSequence):
      sbi = d.getImage(int(i))
      dateTime = sbi.getStartTime()
  elif isinstance(d, SingleBandedImage):
      dateTime = d.getStartTime()

  year = (int)(dateTime.formattedString("yyyy", tz))
  month = (int)(dateTime.formattedString("MM", tz))
  day = (int)(dateTime.formattedString("dd", tz))
  
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes

  
  tertr = TerrainTriangulation(6378169, 295.488065897)
    
  #Initialize the solar zenith angle (SZA) algorithm
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  solzenithalg = suntr.getZenithAngleAlgorithm()
  solazimuthalg = suntr.getAzimuthAlgorithm()

  #Initialize the satellite zenith angle (VZA) algorithm
  sattr = SatelliteTriangulation(tertr, 0, nadirLongitude, nadirPixelSize, nadirAltitude)
  satzenithalg = sattr.getZenithAngleAlgorithm()
  satazimuthalg = sattr.getAzimuthAlgorithm()

  lineValuesA = d.getFloats(0)

  #Loop through image pixels and do the guts, but before that first declare the params object
  params = zeros(2,"d")
  for i in xrange(len):
    #fill the params object with latitude and longitude coordinates in decimal degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
    #print "latlon: ", params
   
    #calculate solar angles
    SZA = solzenithalg.calculate(params)
    SZA = deg2rad * SZA
    
    #calculate sun_azimuth_angle
    SAA = solazimuthalg.calculate(params)
    SAA = deg2rad * SAA

    #calculate satellite angles
    VZA = satzenithalg.calculate(params)
    VZA = deg2rad * VZA
    
    #calculate satellite_azimuth_angle
    VAA = satazimuthalg.calculate(params)
    VAA = deg2rad * VAA

    #calculate co-scaterring angle (CZA) acoording to Dagestad (2003):
    CZA = math.acos(math.cos(SZA) * math.cos(VZA) + math.sin(SZA) * math.sin(VZA) * math.cos(SAA-VAA))
    
    lineValuesA[0][i] = rad2deg * CZA
  # set the new range values, and
  newd.setSamples(lineValuesA)
  # finally return the new data object
  return newd

# Time series functions (call this jython function for image sequences)

def getCoscaterringAngleImageSeqN52(d,user_nadirPixelSize,user_nadirLongitude,user_nadirAltitude):
  import ucar.unidata.data.grid.GridUtil as gu
  nadirPixelSize = Double(user_nadirPixelSize)
  nadirLongitude = Double(user_nadirLongitude)
  nadirAltitude  = Double(user_nadirAltitude)
  if (gu.isTimeSequence(d)):
     newA = d.clone()
     for t in range(d.getDomainSet().getLength()):
         seqA = d.getSample(t)
         newA.setSample(t,getCoscaterringAngleImageN52(seqA,nadirPixelSize,nadirLongitude,nadirAltitude))
     return newA
  else:
     return getCoscaterringAngleImageN52(d,nadirPixelSize,nadirLongitude,nadirAltitude)

##########################################################################################################################################

#################################################
# Raul's work as of 01-02-2006
#################################################
#################################################################################################################################
#Clear sky radiation using Heliosat-2 method (Hammer et al, 2003)
##################################################################################################################################
import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');

from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence 
from visad.meteorology import SingleBandedImage
sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation 
from org.n52.math import SatelliteTriangulation 
from org.n52.math import SunTriangulation 
from jarray import zeros 
from java.lang import Float

def getRadHeliosatN52image(d):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  #rangeValues = d.getFloats()
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  
  #Clone the incoming data object
  newd = d.clone()
  
  
  tertr = TerrainTriangulation(6378169, 295.488065897)
  tz = TimeZone.getTimeZone("GMT");
  if isinstance(d, ImageSequence):
      sbi = d.getImage(int(i))
      dateTime = sbi.getStartTime()
  elif isinstance(d, SingleBandedImage):
      dateTime = d.getStartTime()

  year = (int)(dateTime.formattedString("yyyy", tz))
  month = (int)(dateTime.formattedString("MM", tz))
  day = (int)(dateTime.formattedString("dd", tz))
  
   # julian Day
  feb=28
  if ((((year % 4) == 0) and not((year % 100) == 0)) or ((year % 400) == 0)):
    feb = 29
  months = [31,feb,31,30,31,30,31,31,30,31,30,31]
  monthdays = 0
  for i in range(0,month-1):
    monthdays = monthdays + months[i]
  julianday = day + monthdays

  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  zenithalg = suntr.getZenithAngleAlgorithm()
  
  #Eccentricity correction factor(Eo) and day angle(da)
  
  da = (2*3.1415926*(julianday - 1)/365)
  
  Eo = 1.00011 + 0.034221 * math.cos(da) + 0.00128 * math.sin(da) + 0.00079 * math.cos(2 * da) + 0.000077 * math.sin(2 * da)  
  # Also for 
  
  lineValuesA = d.getFloats()
  #Loop through image pixels and do the guts, but before that first declare the params object
  params = zeros(2,"d")
  for i in xrange(len):
    #fill the params object with latitude and longitude coordinates in decimal degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
   
    #calculate solar angles
      
    SoZA = zenithalg.calculate(params)
    SoZAR = deg2rad * SoZA 

    # Air mass(m) by Kasten and Young(1989)
    x = math.cos(SoZAR)  + 0.50572 *  math.pow((96.07995-SoZAR),-1.6364) #from Hammer paper
    #x = math.sin(SoZA) + 0.50572 * (57.29578*SoZA + 6.07995) # From ESRA atlas
    ms = 0.9669 / x #from Hammer paper
    #ms = (1 - 331/10000)/math.pow(x,-1.6364)
    #Rayleigh optical thickness (ro)  
    if (ms < 20):
        ro = 1/(6.6296 + 1.7513 * ms - 0.1202 * ms * ms + 0.0065 * ms * ms * ms - 0.00013 * ms * ms * ms * ms)  
    else:
        ro = 1/(10.4 + 0.718 * ms)

    # Linke Turbidity 
    try:
        Tl= 1/(ro*ms)*math.log(1367/(1367 * Eo * math.cos(SoZAR)))  #from Hammer paper?
    except:
        Tl=22.76+3.071*0.195476876-27.78 * 0.73  # From the european solar radiation atlas Page 123

    #Clear sky direct Irradiance(Gd) by Page (1996)
    a =-0.8662 * Tl * ro * ms
    Gd = 1367 * Eo * math.exp(a)# e=2.7182818284590451

    # Diffuse irradiance using an empirical relationship by Dumortier 
    Gdf = 1367 * Eo * math.cos(SoZAR) * Eo * (0.0065 + (-0.045 + 0.0646 * Tl )* math.cos(SoZAR) + (0.014 - 0.0327 * Tl)* math.pow(math.cos(SoZAR),2)) # Check the extraterrestrial irradiance 
    
    # Clear sky irradiance Gcl
    Gcl = Gd * math.cos(SoZAR) + Gdf
     
    lineValuesA[0][i] = Gcl
  # set the new range values, and
  newd.setSamples(lineValuesA)
  # finally return the new data object
  return newd

# Time series functions (call this jython function for image sequences)

def getRadHeliosatN52imageSeq(imagelist):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(imagelist)):
     newA = imagelist.clone()
     for t in range(imagelist.getDomainSet().getLength()):
         seqA = imagelist.getSample(t)
         newA.setSample(t,getRadHeliosatN52image(seqA))
     return newA
  else:
     return getRadHeliosatN52image(image)

##################################################################################################################################
# Global (Solar) Irradiatince using Heliosat-2 method
##################################################################################################################################
import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');

from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence 
from visad.meteorology import SingleBandedImage
sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation 
from org.n52.math import SatelliteTriangulation 
from org.n52.math import SunTriangulation 
from jarray import zeros 
from java.lang import Float

def getGlobalIrrHeliosatN52image(d,e):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  #rangeValues = d.getFloats()
  # Also visible reflectivity
  lineValuesA = d.getFloats() #Clear sky radiation (getRadHeliosatN52imageSeq)
  lineValuesB = e.getFloats() #Cloud index (getCLISBseq)
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  #Clone the incoming data object
  newd = d.clone()
  params = zeros(2,"d")
  for i in xrange(len):  
    #Actual global Radiation (Act)
    Act = lineValuesA[0][i] * lineValuesB[0][i]
    
    value = Act
    lineValuesA[0][i] = value
  newd.setSamples(lineValuesA)
  return newd

# Time series functions (call this jython function for image sequences)

def getGlobalIrrHeliosatN52imageSeq(Clear_sky_radiation,Cloud_index):
  import ucar.unidata.data.grid.GridUtil as gu
  d = Clear_sky_radiation
  e = Cloud_index
  if (gu.isTimeSequence(d)):
     newA = d.clone()
     for t in range(d.getDomainSet().getLength()):
         seqA = d.getSample(t)
         seqB = e.getSample(t)
         newA.setSample(t,getGlobalIrrHeliosatN52image(seqA, seqB))
     return newA
  else:
     return getGlobalIrrHeliosatN52image(d,e)

def getCumGlobalIrrHeliosatN52image(d,e,f,t):
  domain = d.getDomainSet()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  #rangeValues = d.getFloats()
  # Also visible reflectivity
  lineValuesA = d.getFloats() #Clear sky radiation (getRadHeliosatN52imageSeq)
  lineValuesB = e.getFloats() #Cloud index (getCLISBseq)

  #Clone the incoming data object
  newd = d.clone()
  for i in xrange(len):
    if t==0:
      previous = 0
    else:
      previous = lineValuesA[0][i]
    current = lineValuesB[0][i] * f * 1.E-6
    #print "previous: ",previous
    #print "current: ",current
    value = previous + current
    lineValuesA[0][i] = float(value)
  newd.setSamples(lineValuesA)
  return newd

# Time series functions (call this jython function for image sequences)

def getCumGlobalIrrHeliosatN52imageSeq(imageSeq):
  from java.util import TimeZone
  tz = TimeZone.getTimeZone("GMT");
  d = imageSeq
  period = 0
  if isinstance(d, ImageSequence):
     sbi = d.getImage(0)
     dateTimeList = []
     dateTimeList = d.getImageTimes()
     #print "dateTimeList: ",dateTimeList
     dateTime = sbi.getStartTime()
     oldday = (int)(dateTime.formattedString("dd", tz))
     print "oldday: ",oldday
     newA = d.clone()
     for t in range(d.getDomainSet().getLength()):
         sbi = d.getImage(t)
         dateTime = sbi.getStartTime()
         newday = (int)(dateTime.formattedString("dd", tz))
         print "newday: ", newday
         if newday == oldday:
           if t == 0:
             i = 0
             seqA = newA.getSample(t)
             period = dateTimeList[t+1] - dateTimeList[t]
           else:
             seqA = newA.getSample(t-1)
             period = dateTimeList[t] - dateTimeList[t-1]
             i = 1
           seqD = d.getSample(t)
           print "period: ",period
           #if period > 900:
           #  period = 900
           newA.setSample(t,getCumGlobalIrrHeliosatN52image(seqA, seqD, period, i))
         else:
           oldday = newday
           i = 0
           seqA = newA.getSample(t)
           seqD = d.getSample(t)
           period = dateTimeList[t+1] - dateTimeList[t]
           print "period: ",period
           #if period > 900:
           #  period = 900 
           newA.setSample(t,getCumGlobalIrrHeliosatN52image(seqA, seqD, period, i))       
     return newA
  else:
     return d
##########################################################################################################################################
# Cloud index using single band
##########################################################################################################################################
import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');

from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence 
from visad.meteorology import SingleBandedImage

sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation, SatelliteTriangulation, SunTriangulation 

from jarray import zeros 
from java.lang import Float

def getCloudIndexImageN52(d):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  rangeValues = d.getFloats()
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  #Clone the incoming data object
  newd = d.clone()
  
  
  tertr = TerrainTriangulation(6378169, 295.488065897)
  tz = TimeZone.getTimeZone("GMT");
  if isinstance(d, ImageSequence):
      sbi = d.getImage(int(i))
      dateTime = sbi.getStartTime()
  elif isinstance(d, SingleBandedImage):
      dateTime = d.getStartTime()

  year = (int)(dateTime.formattedString("yyyy", tz))
  month = (int)(dateTime.formattedString("MM", tz))
  day = (int)(dateTime.formattedString("dd", tz))
  
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes
    
  #Initialize the solar zenith angle (SZA) algorithm
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  solzenithalg = suntr.getZenithAngleAlgorithm()

  #Initialize the satellite zenith angle (VZA) algorithm
  sattr = SatelliteTriangulation(tertr, 0, 0, 3, 35785831)
  satzenithalg = sattr.getZenithAngleAlgorithm()
    
  # julian Day
  feb=28
  if ((((year % 4) == 0) and not((year % 100) == 0)) or ((year % 400) == 0)):
    feb = 29
  months = [31,feb,31,30,31,30,31,31,30,31,30,31]
  monthdays = 0
  for i in range(0,month-1):
    monthdays = monthdays + months[i]
  julianday = day + monthdays

  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  #suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(hour))
  zenithalg = suntr.getZenithAngleAlgorithm()
     
  lineValuesA = d.getFloats() # Ref0.8
  #Loop through image pixels and do the guts, but before that first declare the params object
  params = zeros(2,"d")

  array = zeros(2,"d")

  bin = zeros(2,"d")

  for i in xrange(len):
    #fill the params object with latitude and longitude coordinates in decimal degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
   
    #calculate solar angles
    SoZA = solzenithalg.calculate(params)
    SoZAR = deg2rad * SoZA
    
    #calculate Satellite angle
    VZA = satzenithalg.calculate(params)
    VsZA = deg2rad * VZA

    # CoScaterring angle
    #AZ = VsZA + SoZAR
    #C = math.acos(math.cos(SoZAR) * math.cos(VsZA) + math.sin(SoZAR) * math.sin(VsZA) * math.cos(math.pi - AZ))  

    #Reflectivity of the ground shape(sr)
    #sr = 1 - 0.07261 * C + 0.005187 * math.pow(C,2) + 0.0000778 * math.pow(C,3)#Dano + Bontioli
    #sr = 1  - 0.120505992* C + 0.005326232 * math.pow(C,2) - 0.0000599201065246338 *math.pow(C,3)#Bontioli

    #Reflectivity of ground(sg)
    sg = sr * 0.1502

    #Reflectivity of cloud
    sc = lineValuesA[0][i]/100
    
    # Cloud index(n)  rho_cloud =0.81 Degestad advise 
    #n = (lineValuesA[0][i] * 0.01  - 0.17) / (0.81-0.17) # 0.17 is from weather station data   
    n = (sc - sg) / (0.81 - sg)
    if n <= -0.2:
        #print "n <= -0.2"
        K = 1.2
        #print "n: ",n," K: ", K
    elif n > 1.1:
        #print "n >= 1.1"
        K = 0.05
        #print "n: ",n," K: ", K
    elif n > 0.80:
        #print "n > 0.80"
        K = 2.0667 - 3.6667 * n + 1.6667 * n * n
        #print "n: ",n," K: ", K
    else: 
        #print "n <= 0.80"
        K = 1 - n
        #print "n: ",n," K: ", K

#    if (n <= -0.2):
#        K = 1.2
#    elif (-0.2  < n <= 0.80): 
#        K = 1 - n
#    elif (0.8 < n <= 1.10):
#        K = 2.0667 - 3.6667 * n + 1.6667 * n * n
#    else:
#        K = 0.05

    #Correction of k#new Kch = Kch � 0.01(8 TST � 104)
    Kc = K - 0.01 * (8 * utc - 104)
    if Kc > 1.2:
       Kcr = 1.2
    elif Kc < 0.05:
       Kcr = 0.05
    else:
       Kcr = Kc
    
    lineValuesA[0][i] = Kcr
  # set the new range values, and
  newd.setSamples(lineValuesA)
  # finally return the new data object
  return newd

# Time series functions (call this jython function for image sequences)

def getCloudIndexImageSeqN52(d):

  import ucar.unidata.data.grid.GridUtil as gu

  if (gu.isTimeSequence(d)):
     newA = d.clone()
     for t in range(d.getDomainSet().getLength()):
         seqA = d.getSample(t)
         newA.setSample(t,getCloudIndexImageN52(seqA))
     return newA
  else:
     return getCloudIndexImageN52(d)

##################################
# End Raul's work
##################################

##########################################################################################################################################################
# computing a Normalized Difference Vegetation Index using N52 framework operations
##########################################################################################################################################################
#
# Purpose: Normalized Difference Vegetation Index
# Creator: V. Venus
# Usable in: Images, Data Probe
# Grouped under: User-satellite land surface obs
# see functional describtion of NDVI algorithm at:
# http://adde.itc.nl/IDV/operation/org/n52/operation/NdviAlgorithm.html

sys.add_package('org.n52.processing.ndvi');
from org.n52.processing.ndvi import NdviAlgorithm
from jarray import zeros

def demo_getNdviImageProcessingN52(a,b):
  NdviAlgorithm.main(None) # This line shows the main form (provided the JAR has been signed and given full permissions in the JAVA webstart jnlp)
  lineValuesA = a.getFloats()
  lineValuesB = b.getFloats()
  alg = NdviAlgorithm()
  inp = alg.input
  #params = zeros(2,"d")
  for i in xrange(a.getDomainSet().getLength()):
    inp.low = lineValuesA[0][i]
    inp.high = lineValuesB[0][i]
    value = alg.calculate()
    lineValuesA[0][i] = value
  newd = a.clone()
  newd.setSamples(lineValuesA)
  return newd

##########################################################################################################################################################
# The following procedures are to show how the various functions in relation to the dimension "time" are used
##########################################################################################################################################################
# Purpose: 
# Creator: V. Venus
# 
#
# 
#

def getImageTime(imagesequence, i):
   import sys;
   sys.add_package('visad.meteorology');
   sys.add_package('GridUtil');

   from java.util import TimeZone
   from visad import DateTime
   from visad.meteorology import ImageSequence
   from visad.meteorology import SingleBandedImage

   lineValuesA = imagesequence.getFloats()
   newd = imagesequence.clone()
   tz = TimeZone.getTimeZone("GMT");

   if isinstance(imagesequence, ImageSequence):
       sbi = imagesequence.getImage(int(i))
       dateTime = sbi.getStartTime()
   elif isinstance(imagesequence, SingleBandedImage):
       dateTime = imagesequence.getStartTime()
   for i in xrange(imagesequence.getDomainSet().getLength()):
      hour = float(dateTime.formattedString("H", tz))
      minutes = float(dateTime.formattedString("mm", tz))/60
      time = hour + minutes
      lineValuesA[0][i] = time
   newd.setSamples(lineValuesA)
 
   return newd

##########################################################################################################################################################
# some examples 2Dset calculations
##########################################################################################################################################################
# Purpose: Example function for bandmath, here two grids are substracted
# Creator: V. Venus


def subtractGridded2DSetLayers(a,b):
  temp = a - b
  diff = math.pow(temp,4)
  return diff


##########################################################################################################################################################
# example of how to compute the maximum rate of assimilation using N52 framework processing algorithms
##########################################################################################################################################################
#
# Purpose: Calculates the maximum rate of assimilation on the basis of the actual daytime temperature and the reference temperature. 
# The reference temperature, which is the average daytime temperature of the last ten days, is calculated by ReferenceAndMaintenanceTemperatureAlgorithm.
#
# For C3 plants, the first assimilate is C3H6O3. For C4 plants, the first assimilate is C4H8O4.
#
# Creator: V. Venus
# Usable in: Images, Data Probe
# Grouped under: User-satellite land surface obs
# see functional describtion of algorithm at:
# http://adde.itc.nl/IDV/n52/processing/nl/itc/cropspecificproductionlevels/MaximumAssimilationAlgorithm.html

sys.add_package('nl.itc.cropspecificproductionlevels');
from nl.itc.cropspecificproductionlevels import MaximumAssimilationAlgorithm
from nl.itc.cropspecificproductionlevels.CropInformation import C3C4Crop
from java.lang import Double

def getMaximumAssimilationExampleN52(a,b,c):
  alg = MaximumAssimilationAlgorithm()
  panel = AlgorithmPanel(alg) # This line shows the main algorithm form as a panel so that it may be added to a Jython Display Contrtol (JPanel)
  MaximumAssimilationAlgorithm.main(None) # This line shows the main form (provided the JAR has been signed and given full permissions in the JAVA webstart jnlp
  inp = alg.input
  inp.c3C4CropType = C3C4Crop.C4 #Stores whether this crop is a C3 plant or a C4 plant. Note! This is a more readable version of setC3C4CropType()  
  inp.daytimeTemperature = Double(a)
  inp.referenceTemperature = Double(b)
  out = alg.calculate()
  print out.maximumAssimilation
  return

#########################################################################################################################################################
# compute the satellite viewing angle (SVA), also known as satellite nadir angle, and convert into # degrees.
################################################################################################################################################################################################################################################
#
# Function getSatViewAngle(image) : Float
#
# Creator(s): V. Venus, J.Hendrikse, P. Budde
# Created: 03-10-03
# Modified: 30-10-03
# Contact: venus@itc.nl
# Syntax
#
# getSatViewAngle(image)
#
# Input
# 
# WARNING!
# First:
# XY-coordinates of vector maps are first converted to LatLons, using the projection of the coordinate system of the vector map. 
# XY-coordinates of raster maps are first converted to LatLons, using the projection of the coordinate system of the georeference used by the raster map. 
#
# Coordinate a is an (X,Y) coordinate in LatLons
# Domain type: a coordinate system 
# Coordinate b is an (X,Y) coordinate 
# Domain type: a coordinate system 
#
# Output
#
# getSatViewAngle returns: a value 
# Domain: system domain Value
# Notes:
#
# The coordinate system(s) of the input coordinates are ignored. 
# Spherical distances are calculated over the sphere instead of in a plane.
#
# Usable in
#
def getSatViewAngleTest (d):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
#Note: we want a copy of the range values so we don't pass in 0 (false) here
  rangeValues = d.getFloats()
#Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
#Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
#Clone the incoming data object
  newd = d.clone()
#The at nadir position of the satellite is asumed constant for the time being:
#Meteosat-8 nadir position (reprojected): 0.00904604,-0.00898544
  latNadir = 0.0
  lonNadir = 0.0
  # fill the data object with latitude and longitude coordinates in radians 
  latRadNadir = deg2rad * latNadir   #Ya:=DEGRAD(CRDY(Coordinate_a));
  lonRadNadir = deg2rad * lonNadir   #Xa:=DEGRAD(CRDX(Coordinate_a));
  for i in xrange(len):
    # fill the data object with latitude and longitude coordinates in radians 
    latPixel = latlon[0][i]
    lonPixel = latlon[1][i]
    latRadPixel = deg2rad * latPixel  #Yb:=DEGRAD(CRDY(Coordinate_b))
    lonRadPixel = deg2rad * lonPixel  #Xb:=DEGRAD(CRDX(Coordinate_b))
    # Compute c expressed in radians (Snyder, 1987):
    # c:=ASIN(SQRT(SQ(SIN((Yb-Ya)/2))+COS(Ya)*COS(Yb)*SQ(SIN((Xb-Xa))*2.0; (ILWIS)
    c = math.asin(math.pow(math.sin(latRadPixel-latRadNadir/2),2)+math.cos(latRadNadir)*math.cos(latRadPixel)*math.pow(math.sin(lonRadPixel-lonRadNadir),2)) * 2.0
    # compute the spherical distance in meters (L) over a great circle arc (less than 180 degrees = half the earth's circumference or c < PI) ,
    # which tales the following form: L = c * 6371007.0;
    R = 6371007.0
    L = c * R
    L = L * 0.001 # in kilometers (km)
    # compute the satellite viewing angle (SVA), also known as satellite nadir angle, and convert into degrees.
    SVA = rad2deg * math.atan(L/35785.831) #35785.80 is the altitude of the satellite (Meteosat-8) in km.
    rangeValues[0][i] = SVA
#Set the new range values
  newd.setSamples(rangeValues)
#return the new data object, the satellite viewing angle in radians
  return newd
#
# References
# Snyder, J.P., 1987. Map Projections - A working manual, p.30, USGS Washington, United States of America.
#
#########################################################################################################################################################
# compute satellite viewing angle (SVA), also known as satellite nadir angle, using N52 framework operations
################################################################################################################################################################################################################################################
#
# Function getSatViewAngleN52test(image) : Float
#
# Creator(s): V. Venus, J.Hendrikse, P. Budde
# Creator: V. Venus, J. Hendrikse
# Usable in: Images, Data Probe
# Grouped under: User-satellite land surface obs
# see functional describtion of algorithm at:
# http://adde.itc.nl/IDV/operation/org/n52/operation.html

sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation
from org.n52.math import SatelliteTriangulation
from jarray import zeros

def getSatViewAngleN52test(d):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
#Note: we want a copy of the range values so we don't pass in 0 (false) here
  rangeValues = d.getFloats()
#Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
#Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
#Clone the incoming data object
  newd = d.clone()

  tertr = TerrainTriangulation(6378169, 295.488065897)
  sattr = SatelliteTriangulation(tertr, 0, 0, 3, 35785831)
  zenithalg = sattr.getZenithAngleAlgorithm();

  params = zeros(2,"d")
  for i in xrange(len):
    # fill the data object with latitude and longitude coordinates in degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
    VZA = zenithalg.calculate(params)
    value = math.asin(math.sin(VZA*math.pi/180)/6.65)*rad2deg
    rangeValues[0][i] = value
  newd.setSamples(rangeValues)
  return newd

##################################################################################################################################################################################################################################################
# Land surface temperature (MSG/SEVIRI 4-channel algorithm Donglian) with Cloud Mask
##############################################################################################################################################################################################################################################@####

from java.lang import Float

def getadvSplitWinLST_AllDayCLM(T11,T12,T39,T87,GLC,CLM):

  domain = T11.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  rangeValues = T11.getFloats()
  # Also for T11, T12, T3.9, T8.7, GLC, and CLM (CLM depends on channels T8.7, T9.7, T10.8, T12.8 and T13.4)
  lineValuesT11 = T11.getFloats() #T11
  lineValuesT12 = T12.getFloats() #T12
  lineValuesT39 = T39.getFloats() #T3.9
  lineValuesT87 = T87.getFloats() #T8.7
  lineValuesGLC = GLC.getFloats() #GLC
  lineValuesCLM = CLM.getFloats() #Cloud Layer Mask, where 1=Cloud and 0=clear sky
  
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  #Clone the incoming data object
  newd = T11.clone()


  tertr = TerrainTriangulation(6378169, 295.488065897)
  sattr = SatelliteTriangulation(tertr, 0, 0, 3, 35785831)
  zenithalg = sattr.getZenithAngleAlgorithm();

  params = zeros(2,"T11")
  for i in xrange(len):
    # fill the data object with latitude and longitude coordinates in degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
    VZA = zenithalg.calculate(params) #satellite zenith angle
    VVA = math.asin(math.sin(VZA*math.pi/180)/6.610674631) #satellite-viewing angle

    # Coefficients for the four-channel daytime algorithm :
    x = [-22.0814, 1.0814, 2.2893, 0.1548, 0.2231, 405.7598, -0.00001, -20.3907, 1.0719, 2.6695, -0.0192,   0.3732, 406.9285, -0.00006, -26.2765, 1.0955, 2.2214, -0.1240, 0.8902, 331.8644, -0.00007, 4.1951, 2.2883,  -1.4199,  4.1236, 0.5705, 221.0204, -1.2434, -21.8708,   1.0772,   2.4627, -0.0364, 0.5027, 398.2049, 0.00057, -24.2113,   1.0872, 2.8053, 0.1559, -0.0113, 448.2403,  0.00088, -24.8435,  1.0888, 2.6812, -0.0241, 0.2563, 446.4928, 0.00006, -13.0879, 1.0645,  0.7628, -0.0060,   0.0932, 246.8655, 0.00039, -23.5439, 1.0955, 2.0301,  -0.0384,   0.2725, 323.6286, 0.0005, -23.9135, 1.0891,   2.3913,  -0.0300, 0.6885, 337.0811, 0.00033, -23.0139,   1.0839,   2.3772, -0.0620, 0.7999, 335.4042, 0.00007, -21.9462, 1.0776, 2.6429, 0.0044, 0.5170, 340.9291, -0.00007, -19.0341, 1.0726, 2.2138, -0.0075, 0.3460, 343.9272, 0.00027]
    # Coefficients for the four-channel nighttime algorithm :
    x = [-22.8035, 1.0884, 1.8823, 0.3357, 0.2987, 350.8949, -22.7543,   1.0907, 1.8862, 0.4184,  0.3198, 363.5047,   -3.837,   1.0068, -1.4470, -0.1766, 0.4803,  5.2194, 0.2181,   1.0468,  -0.8622, 2.3312, 0.3009, 265.9501, 32.3526, 0.9289,  -3.0455,  2.3989,   2.0019, 129.5428, -24.8056,  1.1165, 1.2838,   1.0085,  0.0132, 405.8485, -26.4230, 1.1073, 1.5419,  0.6305,   0.2045, 394.3541, -14.0548, -10.054, 1.6736, -0.4148, 0.3697, 260.4226, -24.7400, 1.0971,  2.5880, -0.1936,  0.7697, 316.1983, -24.0435,  1.0934,   1.9820, 0.1464, 0.5767, 323.6190, -22.7214,   1.0873, 1.6851,   0.1604, 0.9131, 267.3659, -23.6807, 1.0943,   1.3240,  0.4622,   0.6197, 253.8679, -21.1784,  1.0844, 2.0485,   0.1141,  0.3397, 342.6013]
        
    landClass = int(lineValuesGLC[0][i])
    if (landClass > 12):
        x1 = 0
        x2 = 0
        x3 = 0
        x4 = 0
        x5 = 0
        x6 = 0
	x7 = 0
        
    else:
        x1 = x[7*landClass]
        x2 = x[7*landClass + 1]
        x3 = x[7*landClass + 2]
        x4 = x[7*landClass + 3]
        x5 = x[7*landClass + 4]
        x6 = x[7*landClass + 5]
        x7 = x[7*landClass + 6]

 
    CM = int(lineValuesCLM[0][i])
    if (CM > 0):
        value = Float.NaN	        
    else:
        value =  (x1 + x2  * lineValuesT11[0][i] + x3 * (lineValuesT11[0][i] - lineValuesT12[0][i]) - x4 * (lineValuesT39[0][i] - lineValuesT87[0][i]) + x5 * ((lineValuesT11[0][i] - lineValuesT12[0][i]) * (lineValuesT11[0][i] - lineValuesT12[0][i])) + x6 * ((1/math.cos(SVA))-1) + x7*lineValuesT39[0][i] * math.cos(SZA)) 

    lineValuesT11[0][i]  = value
  # set the new range values, and
  newd.setSamples(lineValuesT11)
  # finally return the new data object
  return newd

# Time series functions
def getadvSplitWinLST_SeqAllDayCLM(d,e,f,g,h,i):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(d)):
     newA = d.clone()
     for t in range(d.getDomainSet().getLength()):
         subD = d.getSample(t)
         subE = e.getSample(t)
         subF = f.getSample(t)
         subG = g.getSample(t)
         subI = i.getSample(t)
        
         newA.setSample(t,getadvSplitWinLST_AllDayCLM(subD, subE, subF, subG, h, subI))
     return newA
  else:
     return getadvSplitWinLST_AllDayCLM(d,e,f,g,h,i)

##########################################################################################################################
# Precipitable water vapuor by Chester et al. (1987)
##########################################################################################################################

import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');

from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence
from visad.meteorology import SingleBandedImage

sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation
from org.n52.math import SunTriangulation
# until here modified
from jarray import zeros

def getPW(d,e):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  rangeValues = d.getFloats()
  # Also for chanel 10.8 and 12
  lineValuesA = d.getFloats()
  lineValuesB = e.getFloats()
  
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  #Clone the incoming data object
  newd = d.clone()

  tertr = TerrainTriangulation(6378169, 295.488065897)
  tz = TimeZone.getTimeZone("GMT");
  if isinstance(d, ImageSequence):
      sbi = d.getImage(int(i))
      dateTime = sbi.getStartTime()
  elif isinstance(d, SingleBandedImage):
      dateTime = d.getStartTime()

  year = dateTime.formattedString("yyyy", tz)
  month = dateTime.formattedString("mm", tz)
  day = dateTime.formattedString("dd", tz)
  #hour = dateTime.formattedString("hhhh", tz)
  #hour = dateTime.formattedString("Hm", tz)
  #suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(hour))
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  zenithalg = suntr.getZenithAngleAlgorithm()

  params = zeros(2,"d")
  for i in xrange(len):
    # fill the data object with latitude and longitude coordinates in degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
    SoZA = zenithalg.calculate(params)
 
# The Precipitable Water
# PW = (1/0.095)[(1/sec SZA)ln[(T11�Ta)/(T12�Ta)]�0.025] and Ta = air temperature (or T11 � 2.2)
    try:
      value=(1 / 0.095) * ( math.cos(SoZA) * math.log((lineValuesA[0][i] - lineValuesA[0][i] + 2.2) / (lineValuesB[0][i] - lineValuesA[0][i] + 2.2)) - 0.025)
    except ZeroDivisionError:
      value=0
    except ValueError:
      value=0

    #value =  ((lineValuesB[0][i] - lineValuesA[0][i] + 2)) - 0.025
    rangeValues[0][i] = value
  # set the new range values, and
  newd.setSamples(rangeValues)
  # finally return the new data object
  return newd



def getPW_TimeSer(d,e):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(d)):
     newA = d.clone()
     for t in range(d.getDomainSet().getLength()):
         subD = d.getSample(t)
         subE = e.getSample(t)
         newA.setSample(t,getPW(subD, subE))
     return newA
  else:
     return getPW(d,e)



##########################################################################################################################################################
# Solar zenith angle (SZA, degrees) based on a image object using N52 framework operations
##########################################################################################################################################################
#
# Purpose: compute Solar zenith angle (SZA, degrees) based on a image object
# Creator: V. Venus, J. Hendrikse
# Usable in: Images, Data Probe
# Grouped under: User-satellite land surface obs
# see functional describtion of algorithm at:
# http://adde.itc.nl/IDV/operation/org/n52/operation.html

import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');

from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence
from visad.meteorology import SingleBandedImage

sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation
from org.n52.math import SunTriangulation
# until here modified
from jarray import zeros
 
def getSunZenAngleImageN52(d):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
#Note: we want a copy of the range values so we don't pass in 0 (false) here
  rangeValues = d.getFloats()
#Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
#Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
#Clone the incoming data object
  newd = d.clone()
 
  tertr = TerrainTriangulation(6378169, 295.488065897)
#  sattr = SatelliteTriangulation(tertr, 0, 0, 3, 35785831)
#  zenithalg = sattr.getZenithAngleAlgorithm();

  tz = TimeZone.getTimeZone("GMT");
  if isinstance(d, ImageSequence):
      sbi = d.getImage(int(i))
      dateTime = sbi.getStartTime()
  elif isinstance(d, SingleBandedImage):
      dateTime = d.getStartTime()

  year = dateTime.formattedString("yyyy", tz)
  month = dateTime.formattedString("MM", tz)
  day = dateTime.formattedString("dd", tz)
  #hour = dateTime.formattedString("hhhh", tz)
  #hour = dateTime.formattedString("Hm", tz)
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  #suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(hour))
  zenithalg = suntr.getZenithAngleAlgorithm()
 
  params = zeros(2,"d")
  for i in xrange(len):
    # fill the data object with latitude and longitude coordinates in degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
    value = zenithalg.calculate(params)
    rangeValues[0][i] = value
  newd.setSamples(rangeValues)
  return newd


##################################################################################################################################
# Narrow-to-broadband Reflectivity for Meteosat-7 Visible Channel from Meteosat-8 Visible Channels 0.6 and 0.8
##################################################################################################################################
import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');
 
from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence 
from visad.meteorology import SingleBandedImage
sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation 
from org.n52.math import SatelliteTriangulation 
from org.n52.math import SunTriangulation 
from jarray import zeros 
from java.lang import Float
 
def getMet7VisN52imageN52(d,e):
  domain = d.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  rangeValues = d.getFloats()
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  #Clone the incoming data object
  newd = d.clone()
  
  
  tertr = TerrainTriangulation(6378169, 295.488065897)
  tz = TimeZone.getTimeZone("GMT");
  if isinstance(d, ImageSequence):
      sbi = d.getImage(int(i))
      dateTime = sbi.getStartTime()
  elif isinstance(d, SingleBandedImage):
      dateTime = d.getStartTime()
 
  year = (int)(dateTime.formattedString("yyyy", tz))
  month = (int)(dateTime.formattedString("MM", tz))
  day = (int)(dateTime.formattedString("dd", tz))
  
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes
    
  #Initialize the solar zenith angle (SZA) algorithm
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  solzenithalg = suntr.getZenithAngleAlgorithm()
 
  #Initialize the satellite zenith angle (VZA) algorithm
  sattr = SatelliteTriangulation(tertr, 0, 0, 3, 35785831)
  satzenithalg = sattr.getZenithAngleAlgorithm()
    
  # julian Day
  feb=28
  if ((((year % 4) == 0) and not((year % 100) == 0)) or ((year % 400) == 0)):
    feb = 29
  months = [31,feb,31,30,31,30,31,31,30,31,30,31]
  monthdays = 0
  for i in range(0,month-1):
    monthdays = monthdays + months[i]
  julianday = day + monthdays
 
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  zenithalg = suntr.getZenithAngleAlgorithm()
 
  #Eccentricity correction factor (Eo) and day angle (da)
  da = (2*3.1415926*(julianday - 1)/365)
  Eo = 1.00011 + 0.034221 * math.cos(da) + 0.00128 * math.sin(da) + 0.00079 * math.cos(2 * da) + 0.000077 * math.sin(2 * da)  
  
  lineValuesA = d.getFloats() # Rad0.6
  lineValuesB = e.getFloats() # Rad0.8
  #Loop through image pixels and do the guts, but before that first declare the params object
  params = zeros(2,"d")
  for i in xrange(len):
    #fill the params object with latitude and longitude coordinates in decimal degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
   
    #calculate solar angles
    SoZA = solzenithalg.calculate(params)
    SoZAR = deg2rad * SoZA
    
    #calculate Satellite angle
    #VZA = satzenithalg.calculate(params)
    #VsZA = deg2rad * VZA
 
    # CoScaterring angle
    #AZ = VsZA + SoZAR
    #C = math.acos(math.cos(SoZAR) * math.cos(VsZA) + math.sin(SoZAR) * math.sin(VsZA) * math.cos(math.pi - AZ))  
    
    #Correction factor( radiance scaterring toward satellite)
    #xa = -0.0426 * (1 / math.cos(VsZA) + 1 / math.cos(SoZAR))
    #xb = 1 - math.exp(xa)
    
    #try:
       #xb = 1 - math.exp(xa)
    #except OverflowError:
       #xb = 0
  
    #xc = math.cos(SoZA) / (math.cos(VsZA) + math.cos(SoZAR)) 
    #rs = 1367 * ((3 * (1 + math.pow(math.cos(C),2)))/(16 * math.pi)) * xc * xb
   
    # Spectral radiance converted to radiance
    #Lvis1 = (lineValuesA[0][i] * 120.45) / (math.pi * 65.2296)
    #Lvis2 = (lineValuesB[0][i] * 63.43) / (math.pi * 73.0127)
       
    # convert units to radiance
    Lvis1 = lineValuesA[0][i]
    Lvis2 = lineValuesB[0][i]
 
    # Radiance to Meteosat-7 (L), see reference 1
    #L = 1.0605 * (4.49459 * lineValuesA[0][i] + 2.36764 * lineValuesB[0][i]) + 0.5909
    L = 1.0605 * (4.49459 * Lvis1 + 2.36764 * Lvis2) + 0.5909

    #Reflectivity(R) adjusted for Meteosat-7 VIS (see I0met on page 34 reference 2) and Reference 3 equqtion 2
    #R =  (math.pi * L) /(Eo * 690.80 * math.sin(1.570796327 - SoZAR)) # - ((math.pi * rs) / (1367 * math.cos(SoZAR)))
    #R =  (math.pi * L) /(Eo * 693.17 * math.sin(1.570796327 - SoZAR)) # - ((math.pi * rs) / (1367 * math.cos(SoZAR)))
 
    R =  (math.pi * L) /(Eo * 693.17 * math.sin(1.570796327 - SoZAR))
 
    #Reflectivity(R) adjusted for Meteosat-8 VIS 
    #R = (math.pi * (L - 51) * 0.56)/(Eo * 690.80 * SoZAR) - math.pi * rs / (1367 * SoZAR)
   
    #Reflectivity of the ground shape(sr)
    #sr = 1 - 0.07261 * C + 0.005187 * math.pow(C,2) + 0.0000778 * math.pow(C,3)#Dano + Bontioli
    #sr = 1  - 0.120505992* C + 0.005326232 * math.pow(C,2) - 0.0000599201065246338 *math.pow(C,3)#Bontioli
    
    #Reflectivity of ground(sg)
    #sg = sr * 0.1502
    
    lineValuesA[0][i] = R
  # set the new range values, and
  newd.setSamples(lineValuesA)
  # finally return the new data object
  return newd
 
# Time series functions (call this jython function for image sequences)

def getMet7VisN52imageSeqN52(R06,R08):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(R06)):
     newA = R06.clone()
     for t in range(R06.getDomainSet().getLength()):
         seqA = R06.getSample(t)
         seqB = R08.getSample(t)
         newA.setSample(t,getMet7VisN52imageN52(seqA,seqB))
     return newA
  else:
     return getMet7VisN52imageN52(R06,R08)
########################################################################################################################################
#References
# 1. http://www.eumetsat.int/idcplg?IdcService=GET_FILE&dDocName=PDF_REP_050050_CALIB_MET-7&RevisionSelectionMethod=LatestReleased)
# 2. Simula
########################################################################################################################################




##################################################################################################################################################################################################################################################
# Surface temperature retrieval from MSG-SEVIRI observations (four channel algorithm)
##############################################################################################################################################################################################################################################@####
# Purpose: compute surface (skin) temperature based on image objects from MSG-SEVIRI
# Autrhor(s): V. Venus, R. Cumba
# Usable displays: Images, Data Probe, 
# Used by: getLSTDayNightImageSeqN52(T11,T12,T39,T87,GLC,CLM) for image sequences (see end of this section)
# Uses: getCloudLayerMaskImageSeqN52 filed under User>satellite atmosphere obs
# Grouped under: User-satellite land surface obs
# see functional describtion of algorithm at:
# http://adde.itc.nl/IDV/operation/org/n52/operation.html
# version: 0.4

import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');

from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence
from visad.meteorology import SingleBandedImage

sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation
from org.n52.math import SatelliteTriangulation
from org.n52.math import SunTriangulation
from jarray import zeros
from java.lang import Float

def getLSTDayNightImageN52(T11,T12,T39,T87,GLC,CLM):

  domain = T11.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) here
  #for T11, T12, T3.9, T8.7, GLC, and CLM (CLM depends on channels T8.7, T9.7, T10.8, T12.8 and T13.4)
  lineValuesT11 = T11.getFloats() #T11
  lineValuesT12 = T12.getFloats() #T12
  lineValuesT39 = T39.getFloats() #T3.9
  lineValuesT87 = T87.getFloats() #T8.7
  lineValuesGLC = GLC.getFloats() #GLC
  lineValuesCLM = CLM.getFloats() #Cloud Layer Mask, where 1=Cloud and 0=clear sky
  
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  #Clone the incoming data object
  newd = T11.clone()

  #Get time from image object
  tz = TimeZone.getTimeZone("GMT");
  if isinstance(T11, ImageSequence):
      sbi = T11.getImage(int(i))
      dateTime = sbi.getStartTime()
  elif isinstance(T11, SingleBandedImage):
      dateTime = T11.getStartTime()
  year = dateTime.formattedString("yyyy", tz)
  month = dateTime.formattedString("MM", tz)
  day = dateTime.formattedString("dd", tz)
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes

  #Intilialize the terrain triangulation
  tertr = TerrainTriangulation(6378169, 295.488065897)

  #Initialize the solar zenith angle (SZA) algorithm
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  solzenithalg = suntr.getZenithAngleAlgorithm()

  #Initialize sunrise and sunset algorithms
  sunrisealg = suntr.getSunRiseAlgorithm() 
  sunsetalg  = suntr.getSunSetAlgorithm()

  #Initialize the satellite zenith angle (VZA) algorithm
  sattr = SatelliteTriangulation(tertr, 0, 0, 3, 35785831)
  satzenithalg = sattr.getZenithAngleAlgorithm()

  #Loop through image pixels and do the guts, but before that first declare the params object
  params = zeros(2,"d")
  for i in xrange(len):
    #fill the params object with latitude and longitude coordinates in decimal degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
   
    #calculate satellite angles
    VZA = satzenithalg.calculate(params) #satellite zenith angle
    VVA = math.asin(math.sin(VZA*math.pi/180)/6.610674631) #The satellite-viewing angle (VVA) relates to VZA as follows:
                                                           #sinVZA = (satellite height + earth radius)/earth radius * sinVVA
                                                           #       = (   35785831      +   6378169   )/  6378169    * sinVVA
                                                           #       = 6.610674631 * sinVVA (or 6.65 check this!!!)

    #retrieve the cover types from the Global Land Cover map (1km) available at http://glcf.umiacs.umd.edu/data/landcover/.          
    landClass = int(round(lineValuesGLC[0][i],0)) #For programmatic simplicity we do not increment the land cover classes with one yet.
                                         #(which would sync the data with the cover classs identifiers as used in the publication)

    #first find out if the imagepixel was observed during night or day and run the appropriate algorithm
    sunrise = sunrisealg.calculate(params)
    sunset  = sunsetalg.calculate(params)
    if (utc < sunrise) and (utc > sunset): #it is night, so use the corresponding nighttime coefficients (i.e. "a(l)"):
        #print "    Current pixel (#", i + 1, ") is in the night."
        # Coefficients for the four-channel nighttime algorithm:
        a = [-22.8035, 1.0884, 1.8823, 0.3357, 0.2987, 350.8949, -22.7543,   1.0907, 1.8862, 0.4184,  0.3198, 363.5047,   -3.837,   1.0068, -1.4470, -0.1766, 0.4803,  5.2194, 0.2181,   1.0468,  -0.8622, 2.3312, 0.3009, 265.9501, 32.3526, 0.9289,  -3.0455,  2.3989,   2.0019, 129.5428, -24.8056,  1.1165, 1.2838,   1.0085,  0.0132, 405.8485, -26.4230, 1.1073, 1.5419,  0.6305,   0.2045, 394.3541, -14.0548, -10.054, 1.6736, -0.4148, 0.3697, 260.4226, -24.7400, 1.0971,  2.5880, -0.1936,  0.7697, 316.1983, -24.0435,  1.0934,   1.9820, 0.1464, 0.5767, 323.6190, -22.7214,   1.0873, 1.6851,   0.1604, 0.9131, 267.3659, -23.6807, 1.0943,   1.3240,  0.4622,   0.6197, 253.8679, -21.1784,  1.0844, 2.0485,   0.1141,  0.3397, 342.6013]
    else:                                  #it is day, so use the corresponding daytime coefficients (i.e. "a(l)"):
        #print "    Current pixel (#", i + 1, ") is in the day."
        # Coefficients for the four-channel daytime algorithm:
        a = [-22.0814, 1.0814, 2.2893, 0.1548, 0.2231, 405.7598, -0.00001, -20.3907, 1.0719, 2.6695, -0.0192,   0.3732, 406.9285, -0.00006, -26.2765, 1.0955, 2.2214, -0.1240, 0.8902, 331.8644, -0.00007, 4.1951, 2.2883,  -1.4199,  4.1236, 0.5705, 221.0204, -1.2434, -21.8708,   1.0772,   2.4627, -0.0364, 0.5027, 398.2049, 0.00057, -24.2113,   1.0872, 2.8053, 0.1559, -0.0113, 448.2403,  0.00088, -24.8435,  1.0888, 2.6812, -0.0241, 0.2563, 446.4928, 0.00006, -13.0879, 1.0645,  0.7628, -0.0060,   0.0932, 246.8655, 0.00039, -23.5439, 1.0955, 2.0301,  -0.0384,   0.2725, 323.6286, 0.0005, -23.9135, 1.0891,   2.3913,  -0.0300, 0.6885, 337.0811, 0.00033, -23.0139,   1.0839,   2.3772, -0.0620, 0.7999, 335.4042, 0.00007, -21.9462, 1.0776, 2.6429, 0.0044, 0.5170, 340.9291, -0.00007, -19.0341, 1.0726, 2.2138, -0.0075, 0.3460, 343.9272, 0.00027]

    if (landClass > 12): #Urban and Built-up are all set zero.
        a0 = 0
        a1 = 0
        a2 = 0
        a3 = 0
        a4 = 0
        a5 = 0
	a6 = 0
        
    else:
        a0 = a[7*landClass]
        a1 = a[7*landClass + 1]
        a2 = a[7*landClass + 2]
        a3 = a[7*landClass + 3]
        a4 = a[7*landClass + 4]
        a5 = a[7*landClass + 5]
        a6 = a[7*landClass + 6]

    #filter for cloud contaminated pixels and do the guts
    if (lineValuesCLM[0][i] < 0):
        CM = 0 #Cloud (Layer) Mask, where 1=cloudy and 0=clear sky
    else:
        CM = lineValuesCLM[0][i]
    if (CM > 0):
        value = Float.NaN #cloud contaminated pixels so fill pixel value with "missing" or NaN	        
    else: #clear-sky so calculate surface temperature

        if (utc < sunrise) and (utc > sunset): #it is night, so run the nighttime algorithm using the appropriate coefficients (i.e. "a(l)")

            value =  a0 + a1*lineValuesT11[0][i] + a2*(lineValuesT11[0][i] - lineValuesT12[0][i]) + a3*(lineValuesT39[0][i] - lineValuesT87[0][i]) + a4*math.pow((lineValuesT11[0][i] - lineValuesT12[0][i]),2) + a5*((1/math.cos(VVA))-1)
 
        else: #it is day, so run the daytime algorithm using the appropriate coefficients (i.e. "a(l)"):

            #calculate solar zenith angle to reduce the solar signal contamination in the brightness temperature, which makes T3.9 decrease again
            SZA = solzenithalg.calculate(params)

            value =  a0 + a1*lineValuesT11[0][i] + a2*(lineValuesT11[0][i] - lineValuesT12[0][i]) + a3*(lineValuesT39[0][i] - lineValuesT87[0][i]) + a4*math.pow((lineValuesT11[0][i] - lineValuesT12[0][i]),2) + a5*((1/math.cos(VVA))-1) + a6*lineValuesT39[0][i]*math.cos(SZA)
            #if (value < 288):
            #   value = Float.NaN #Cloud (Layer) Mask, where 1=cloudy and 0=clear sky
            #elif (value > 343):
            #   value = Float.NaN
            #else:
            #   value = value      
    lineValuesT11[0][i]  = value
  # set the new range values, and
  newd.setSamples(lineValuesT11)
  # Finish with returning the new data object
  return newd

# Time series functions of above (call this jython function for image sequences)
def getLSTDayNightImageSeqN52(T11,T12,T39,T87,GLC,CLM):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(T11)):
     newA = T11.clone()
     seq = T11.getDomainSet().getLength()
     for t in range(seq):
         #print "    Computing LST for image nr. ", t + 1, " of ", seq 
         seqT11 = T11.getSample(t)
         seqT12 = T12.getSample(t)
         seqT39 = T39.getSample(t)
         seqT87 = T87.getSample(t)
         seqCLM = CLM.getSample(t)
        
         newA.setSample(t,getLSTDayNightImageN52(seqT11, seqT12, seqT39, seqT87, GLC, seqCLM))
     return newA
  else:
     return getLSTDayNightImageN52(T11,T12,T39,T87,GLC,CLM)
##################################################################################################################################################################################################################################################
# References:
# Sun, D., Venus, V., and Pinker, R.T., (in preparation). Surface temperature retrieval from MSG-SEVIRI observations. Journal of Geophysical Research-Atmosphere.
# Hansen et al., 2000
##################################################################################################################################################################################################################################################

##################################################################################################################################################################################################################################################
# Cloudy layer mask (CLM)
##################################################################################################################################################################################################################################################

def getCloudLayerMaskImageN52(T87,T97,T11,T12,T13):
  domain = T87.getDomainSet()
  len =  domain.getLength()
  #Clone the incoming data object
  newd = T87.clone()

  # Also for T87, T97, T11, T12, and T13
  lineValuesT87 = T87.getFloats()
  lineValuesT97 = T97.getFloats()
  lineValuesT11 = T11.getFloats()
  lineValuesT12 = T12.getFloats()
  lineValuesT13 = T13.getFloats()

  for i in xrange(len):
   
    # Compute the average of infrared chanels:
    Av = (lineValuesT87[0][i] + lineValuesT97[0][i] + lineValuesT11[0][i] + lineValuesT12[0][i] + lineValuesT13[0][i])/5
   
    # Compute the standard deviation of infrared chanels:
    Sd = math.sqrt((math.pow((lineValuesT87[0][i]-Av),2)+ math.pow((lineValuesT97[0][i]-Av),2) + math.pow((lineValuesT11[0][i]-Av),2) + math.pow((lineValuesT12[0][i]-Av),2) + math.pow((lineValuesT13[0][i]-Av),2))/2)
    
    # Compute the standard deviation multiplied by the average of infrared chanels:
    AvSD = Av * Sd

    #Cloud mask (CM) # where 1=Cloudy and 0=clear sky   
    if (AvSD < 2884.7116): # or 2894.7116
        CM = 1
    else:
        CM = 0
     
    lineValuesT87[0][i] = CM
  # set the new range values, and
  newd.setSamples(lineValuesT87)
  # finally return the new data object
  return newd

# Time series functions (call this jython function for image sequences)

def getCloudLayerMaskImageSeqN52(T87,T97,T11,T12,T13):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(T87)):
     newA = T87.clone()
     seq = T87.getDomainSet().getLength()
     for t in range(seq):
         #print "    Computing Cloud Layer Mask for image nr. ", t + 1, " of ", seq 
         seqT87 = T87.getSample(t)
         seqT97 = T97.getSample(t)
         seqT11 = T11.getSample(t)
         seqT12 = T12.getSample(t)
         seqT13 = T13.getSample(t)
         newA.setSample(t,getCloudLayerMaskImageN52(seqT87,seqT97,seqT11,seqT12,seqT13))
     return newA
  else:
     return getCloudLayerMaskImageN52(T87,T97,T11,T12,T13)

##########################################################
# Temperature model
##########################################################
# Time series functions (call this jython function for image sequences)

def getTempDiurnalImageSeqN52(LST,SingleBand):

  import sys;
  sys.add_package('visad.meteorology');
  sys.add_package('GridUtil');

  from java.util import TimeZone
  from visad import DateTime
  from visad.meteorology import ImageSequence
  from visad.meteorology import SingleBandedImage

  sys.add_package('org.n52.math');
  sys.add_package('org.n52.operation');
  sys.add_package('org.n52.operation.cropmodeling');
  from org.n52.operation.cropmodeling import TemperatureModelAlgorithm

  print "Function getTempDiurnalImage started ..."

  domain = SingleBand.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()

  # Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  # Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  # Clone the incoming data object
  newd = LST.clone()

  # Initialize temperature algorithm

  # Get start time of image sequence
  tz = TimeZone.getTimeZone("GMT");
  if isinstance(LST, ImageSequence):
      sbi = LST.getImage(int(0))
      dateTime = sbi.getStartTime()
  elif isinstance(LST, SingleBandedImage):
      dateTime = LST.getStartTime()
  year = dateTime.formattedString("yyyy", tz)
  month = dateTime.formattedString("MM", tz)
  day = dateTime.formattedString("dd", tz)
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))

  tempModelAlg = TemperatureModelAlgorithm()
  tempModelAlg.setResolution(15)
  tempModelAlg.setStartTime(int(year), int(month), int(day), int(hour), int(minutes))

  #print "Starting for loop ... number of iterations = ", len
  # calculate temperatures for all coordinates (crd) for all times (time)
  for crd in xrange(len):
      #print "Pixel nr. ", crd + 1, " of ", len 
      # feed the algorithm with pixel specific latitude coordinates in decimal degrees 
      tempModelAlg.setLatitude(latlon[0][crd])
      # get length of imageSequence
      seq = LST.getDomainSet().getLength()
      # create array to store temperatures 
      arr = zeros(seq, "d")
      # get all temperatures for one coordinate for all times time and put them into arr
      #print "Entering nested forloop 1"
      for time in xrange(seq):
          # get the values
          # Pass in 0 as false so we don't copy the values
          #print "    Computing image nr. ", time + 1, " of ", seq 
          values = LST.getSample(time).getFloats(0)
          #print "    Value for current pixel (#", crd + 1, "): ",values[0][crd]
          arr[time] = values[0][crd]
      print "Exited nested forloop 1"
      # Calculate the temperatures
      #print "Starting temperature calculation"
      res = tempModelAlg.calculate(arr)
      #print "Finished temperature calculation"
      # feed the temperatures back into the clone newd
      #print "Entering nested forloop 2"
      for time in xrange(seq):
          # Pass in 0 as false so we don't copy the values
          #print "    Wrting image nr. ", time + 1, " of ", seq 
          values = newd.getSample(time).getFloats(0)
          try:
              values[0][crd] = res[time]
          except:
              values[0][crd] = Float.NaN #cloud contaminated pixels cannot be solved so fill pixel value with "missing" or NaN	
          #newd.setSamples(time,values)
      print "Exited nested forloop 2"
  return newd

##########################################################
# MODIS Aerosol optical depth at user-defined wavelength
##########################################################
# produces Aerosol optical depth (AOT) at any wavelength [0.55 - 0.67], 
# [unitless] recommended range 0.05 - 0.5
#
# The AOT for a certain wavelength can be written as (Angstrom, 1929):
# AOT = Angstrom exponent � wavelength^Angstrom�s turbidity coefficient,
#
# but in the case of MODIS we have not the Angstrom coefficients, but the 
# Regression Slope and Intercept for lineair regression between the
# 0.47 and 0.67 microns retrievals by MODIS (AQUA or TERRA):
# 
def makeMODIS2DSetAOT(ObservedAOT1,ObservedAOT2,user_Source_Wavelength_1,user_Source_Wavelength_2,user_Target_Wavelength):

  domain = ObservedAOT1.getDomainSet()
  len =  domain.getLength()
  #Clone the incoming data object
  newd = ObservedAOT1.clone()
  print "Computing AOT for wavelength: ", user_Target_Wavelength

  # get slope,intercept
  lineValuesAOT1 = ObservedAOT1.getFloats()
  lineValuesAOT2 = ObservedAOT2.getFloats()
  logWavelength1 = math.log(float(user_Source_Wavelength_1))
  #logWavelength2 = math.log(float(user_Source_Wavelength_2))
  for i in xrange(len):
    #print "    Computing AOT for image pixel nr. ", i + 1, " of ", len 
    try:
        alpha =  math.log(lineValuesAOT1[0][i]/lineValuesAOT2[0][i])/math.log(float(user_Source_Wavelength_2)/float(user_Source_Wavelength_1))
    except:   
        alpha = 1.3 # commonly employed when missing, natural atmosphere is assumed
    try:  
        beta  =  math.exp(math.log(lineValuesAOT1[0][i]) + alpha * logWavelength1) 
    except:    
        beta  = 0.1 # clean atmosphere is assumed equivalent to 28 km visibility (clean: 0.00=340km, turbid: 0.20=11km, very turbid: 0.4<5km)
    try:
        value =  beta * math.pow(float(user_Target_Wavelength),-alpha)
    #print "    AOT for image pixel nr. ", i + 1, " is: ", value 
    except:
        value = Float.NaN
    lineValuesAOT1[0][i] = value
  newd.setSamples(lineValuesAOT1)
  return newd

##########################################################
# MODIS Total column ozone (O3)
##########################################################
# produces Total column ozone in g m-2, 
# recommended range 6.4 - 9.7l. 
#
# 1 Dobson unit = 0.021416667 g m-2
# MODIS 08 gives: Total Ozone Burden: Mean, units: "Dobson Units"

from java.lang import Double

def makeMODIS2DSetO3(O3Dobson):

  domain = O3Dobson.getDomainSet()
  len =  domain.getLength()
  #Clone the incoming data object
  newd = O3Dobson.clone()

  # do the guts
  lineValuesO3 = O3Dobson.getFloats()

  for i in xrange(len):
    if Double(lineValuesO3[0][i]).isNaN():
        try:
            j = i - 1
            k = i + 1
            values = (lineValuesO3[0][j]+lineValuesO3[0][k]/2) * 0.021416667
            print "    Computing O3 for image pixel nr. ", i + 1, " of ", len, " value: ", ozone 
        except:
            values = 6.4
    else:
        values = lineValuesO3[0][i] * 0.021416667
    lineValuesO3[0][i] = values
  newd.setSamples(lineValuesO3)
  return newd
     
##########################################################
# Helper functions
##########################################################
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');

##########################################################
# Convert image sequences into a flat 2DSet
##########################################################
# Purpose: 
# Creator: V. Venus
# 
def convertImageTo2DSetN52(a):
  lineValuesA = a.getFloats()
  for i in xrange(a.getDomainSet().getLength()):
    value = lineValuesA[0][i]
    lineValuesA[0][i] = value
  newd = a.clone()
  newd.setSamples(lineValuesA)
  return newd

def convertImageTo2DSetSeqN52(a):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(a)):
     newA = a.clone()
     for t in range(a.getDomainSet().getLength()):
         #print "t = ", t
         subA = a.getSample(t)
         newA.setSample(t,convertImageTo2DSetN52(subA))
     return newA
  else:
     return convertImageTo2DSetN52(a)

def resampleGrid(oldGrid, gridwithNewDomain):
   import ucar.unidata.data.grid.GridUtil as gu
   newLocs = gu.getSpatialDomain(gridwithNewDomain)
   return gu.resampleGrid(oldGrid, newLocs)

def convert2integer(a):
  lineValuesA = a.getFloats()
  for i in xrange(a.getDomainSet().getLength()):
    integers = round(lineValuesA[0][i],0)
    lineValuesA[0][i] = integers 
  newd = a.clone()
  newd.setSamples(lineValuesA)
  return newd

#change units from geopotential meters to meters
def getAltitude(z):
   import ucar.visad.quantities.GeopotentialAltitude as ga
   import ucar.visad.quantities.Gravity as gr
   import ucar.unidata.data.grid.GridUtil as gu
   zUnit = gu.getParamType(z).getRealComponents()[0].getDefaultUnit()
   if zUnit.equals(ga.getGeopotentialMeter()):
      z = z.divide(gr.newReal())
   return z

def newUnit(field, varname, unitname):
  from ucar.visad import Util
  from ucar.unidata.data.grid import GridUtil
  newunit = Util.parseUnit(unitname)
  newType = Util.makeRealType(varname, newunit)
  return GridUtil.setParamType(field, newType)

##########################################################
# MODIS Helper Functions
##########################################################
import sys;
sys.add_package('visad');

def makeMODISSwathTo2DSet(a):
  from visad import *
  from visad import RealTupleType
  from visad import FunctionType
  from visad import Gridded2DSet
  from visad import FlatField

  lat = a[0][0]
  lon = a[0][1]
  xyLen = lat.getDomainSet().getLengths()
  values = a[1]
  latVals = lat.getFloats(0)[0]
  lonVals = lon.getFloats(0)[0]
  domain = Gridded2DSet(RealTupleType.SpatialEarth2DTuple,
                       (lonVals, latVals), xyLen[0], xyLen[1],
                       None, None, None, 0, 0 )
  image = FlatField(FunctionType(domain.getType().getDomain(),makeRealType("values")), domain)
  image.setSamples(values.getFloats(0),0)
  return image

def makeMODISGriddedTo2DSet(a):
  from visad import *
  from visad import RealTupleType
  from visad import FunctionType
  from visad import Gridded2DSet
  from visad import FlatField

  domain = a.getDomainSet()
  len =  domain.getLength()

  values = a.getFloats(0)
  lonVals=zeros(len,"f")
  latVals=zeros(len,"f")
  for lat in xrange(180):
    for lon in xrange(360):
      lonVals[lon - lat*360] = lon - 180
      latVals[lon - lat*360] = lat - 90
  #latVals = a.getFloats()[0][1]
  #lonVals = a.getFloats()[0][0]
  domain = Gridded2DSet(RealTupleType.SpatialEarth2DTuple,
                       (lonVals, latVals), 360, 180,
                       None, None, None, 0, 0 )
  image = FlatField(FunctionType(domain.getType().getDomain(),makeRealType("values")), domain)
  image.setSamples(values,0)
  return image

def makeMODISGridded4KMTo2DSet(a):
  from visad import *
  from visad import RealTupleType
  from visad import FunctionType
  from visad import Gridded2DSet
  from visad import FlatField

  domain = a.getDomainSet()
  len =  domain.getLength()

  values = a.getFloats(0)
  lonVals=zeros(len,"f")
  latVals=zeros(len,"f")
  for lat in xrange(180):
    for lon in xrange(360):
      lonVals[lon - lat*360] = lon - 180 #/4319.5 #1/2 of 8639
      latVals[lon - lat*360] = lat - 90  #/2159.5 #1/2 of 4319
  #latVals = a.getFloats()[0][1]
  #lonVals = a.getFloats()[0][0]
  domain = Gridded2DSet(RealTupleType.SpatialEarth2DTuple,
                       (lonVals, latVals), 360, 180,
                       None, None, None, 0, 0 )
  image = FlatField(FunctionType(domain.getType().getDomain(),makeRealType("values")), domain)
  image.setSamples(values,0)
  return image

def makeTOMSGriddedTo2DSet(a):
  from visad import *
  from visad import RealTupleType
  from visad import FunctionType
  from visad import Gridded2DSet
  from visad import FlatField

  domain = a.getDomainSet()
  len =  domain.getLength()

  values = a.getFloats(0)
  lonVals=zeros(len,"f")
  latVals=zeros(len,"f")
  for lat in xrange(180):
    for lon in xrange(288):
      lonVals[lon - lat*288] = lon - 180
      latVals[lon - lat*288] = lat - 90
  #latVals = a.getFloats()[0][1]
  #lonVals = a.getFloats()[0][0]
  domain = Gridded2DSet(RealTupleType.SpatialEarth2DTuple,
                       (lonVals, latVals), 288, 180,
                       None, None, None, 0, 0 )
  image = FlatField(FunctionType(domain.getType().getDomain(),makeRealType("values")), domain)
  image.setSamples(values,0)
  return image

##########################################################################################################################################################
# reclassify MOD12Q1 into AVHRR IGBP Class Scheme 
# (ftp://ftp.glcf.umiacs.umd.edu/glcf/Global_Land_Cover/Asia-Pacific/ap-latlong-1km-landcover/ap-latlong-1km-landcover.glcf > http://duckwater.bu.edu/lc/igbp.txt)  
##########################################################################################################################################################
#
# Purpose: reclassify MOD12Q1 into AVHRR IGBP Class Scheme used by Global Land Cover Facility, University of Maryland
# Creator: V. Venus
# Usable in: Images, Data Probe
# Grouped under: User-satellite land surface obs

def recMOD12Q1(GLC):
  lineValuesA = GLC.getFloats()
  for i in xrange(GLC.getDomainSet().getLength()):
    #retrieve the cover types from the MODIS Global Land Cover map (1km)         
    raw = int(lineValuesA[0][i])
    if raw == 11: #Permanent Wetlands
        landClass = 10 #classify as "Grassland"
        print "    Value for current pixel (",lineValuesA[0][i],") was changed into (#", i + 1, "): ", landClass
    elif raw == 12: #equals "Croplands"
        landClass = 11 #classify as "Cropland"
        print "    Value for current pixel (",lineValuesA[0][i],") was changed into (#", i + 1, "): ", landClass
    elif raw == 14: #equals "Cropland/Natural Vegetation Mosaic"
        landClass = 11 #classify as "Cropland"
        print "    Value for current pixel (",lineValuesA[0][i],") was changed into (#", i + 1, "): ", landClass
    elif raw == 16: #equals "Barren or Sparsely Vegetated"
        landClass = 12 #classify as "Bare Ground"
        print "    Value for current pixel (",lineValuesA[0][i],") was changed into (#", i + 1, "): ", landClass
    elif raw == 17: #equals "IGBP Water Bodies"
        landClass = 0 #classify as "Bare Ground"
        print "    Value for current pixel (",lineValuesA[0][i],") was changed into (#", i + 1, "): ", landClass
    else:
        #print "    Value for current pixel was NOT changed(#", i + 1, "): ",lineValuesA[0][i]       
        landClass = raw
    lineValuesA[0][i] = landClass 
  newd = GLC.clone()
  newd.setSamples(lineValuesA)
  return newd


#####################################
# Here starts Joseph's MSc
#####################################

##########################################################################################################################################################
# Mask Pathfinder SST for erroneous data
##########################################################################################################################################################
# Purpose: 
# Creator: V. Venus, J. Mbui 
# 

def maskSSTPathFinder(SST,CLM):
  try:
    if CLM > 241:
        value = Float.NaN
        print "    Value for current pixel is cloud contaminated, value 'missing' assigned."
    else:
        value = SST
        print "    Value for current pixel was not changed."
  except ValueError:
    value = Float.NaN 
  return value

##########################################################################################################################################################
# convert scaled SeaWiFS values into depth in meters
##########################################################################################################################################################
# Purpose: convert scaled SeaWiFS values into depth in meters
# Creator: V. Venus
# Source:  http://oceancolor.gsfc.nasa.gov/cgi/reefs.pl

def SeaWiFSraw2depthMetersAs2DSetLayers(n):

  sys.add_package('ucar.visad');
  from ucar.visad import Util
  from ucar.unidata.data.grid import GridUtil
  if n == 0:        #no data
     K = Float.NaN
  elif n == 1:      #land
     K = 0
  elif n == 255:    #cloud or other masking condition
     K = Float.NaN
  else:             #median depth
     K = -0.2*exp(6.2146080984221917426367422425949*(n - 2)/252)
  newunit = Util.parseUnit("meter")
  newType = Util.makeRealType("Depth", newunit)
  return GridUtil.setParamType(K, newType)

#####################################
# Here ends Joseph's MSc
#####################################

##########################################################################################################################################################
# Temperature (Dew point / Ambient air) from MODIS using Pressure
##########################################################################################################################################################
# Purpose: 
# Creator: V. Venus, A. Mendez, A. Skidmore
# 
def tempModisRetAtmPressureN52Image(RTP950,RTP1000,PSFC):
  lineValuesRTP950 = RTP950.getFloats()
  lineValuesRTP1000 = RTP1000.getFloats()
  lineValuesPSFC = PSFC.getFloats()
  for i in xrange(RTP950.getDomainSet().getLength()):
    try:
      value = (((lineValuesRTP1000[0][i] - lineValuesRTP950[0][i])/(1000-950))*(lineValuesPSFC[0][i]-1000))+lineValuesRTP1000[0][i]
      print "    Value for current pixel (#", i + 1, "): ",value
    except ValueError:
      value = Float.NaN
      #print "    Value for current pixel is cloud contaminated, value 'missing' assigned."
    except ZeroDivisionError:
      value= Float.NaN
    if value < 223.15:
       value = Float.NaN
    elif value > 315.15:
       value = Float.NaN
    else:
       value = value
    lineValuesRTP950[0][i] = value
  newd = RTP950.clone()
  newd.setSamples(lineValuesRTP950)
  return newd

def tempModisRetAtmPressureN52ImageSeq(RTP950,RTP1000,PSFC):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(RTP950)):
     newA = RTP950.clone()
     seq = RTP950.getDomainSet().getLength()
     for t in range(seq):
         print "    Computing ambient air temperature for image nr. ", t + 1, " of ", seq 
         seqRTP950  = RTP950.getSample(t)
         seqRTP1000 = RTP1000.getSample(t)
         seqPSFC    = PSFC.getSample(t)
         newA.setSample(t,tempModisRetAtmPressureN52Image(seqRTP950,seqRTP1000,seqPSFC))
     return newA
  else:
     return tempModisRetAtmPressureN52Image(RTP950,RTP1000,PSFC)

##########################################################################################################################################################
# Temperature (Dew point / Ambient air) from MODIS using Height
##########################################################################################################################################################
# Purpose: 
# Creator: V. Venus, A. Mendez, A. Skidmore
# 
def tempModisRetAtmHeightN52Image(RTP950,RTP1000,RTA950,RTA1000,ASFC):
  lineValuesRTP950  = RTP950.getFloats()
  lineValuesRTP1000 = RTP1000.getFloats()
  lineValuesRTA950  = RTA950.getFloats()
  lineValuesRTA1000 = RTA1000.getFloats()
  lineValuesASFC    = ASFC.getFloats()
  for i in xrange(RTP950.getDomainSet().getLength()):
    lineValuesRPT950  = lineValuesRTP950[0][i] +0.00975124378109452736318407960199*lineValuesRTA950[0][i]
    lineValuesRPT1000 = lineValuesRTP1000[0][i]+0.00975124378109452736318407960199*lineValuesRTA1000[0][i]
    lineValuesDeltaTemp = abs(lineValuesRPT1000-lineValuesRPT950)
    #print "    lineValuesDeltaTemp for current pixel (#", i + 1, "): ",lineValuesDeltaTemp

    lineValuesDeltaHeightAtm = abs(lineValuesRTA950[0][i]-lineValuesRTA1000[0][i])
    #print "    lineValuesDeltaHeightAtm for current pixel (#", i + 1, "): ",lineValuesDeltaHeightAtm

    lineValuesDeltaHeightAtmSfc = abs(lineValuesRTA1000[0][i]-lineValuesASFC[0][i])
    #lineValuesDeltaHeightAtmSfc = abs((lineValuesRTA1000[0][i]-float(ASFC))-2)
    #print "    lineValuesDeltaHeightAtmSfc for current pixel (#", i + 1, "): ",lineValuesDeltaHeightAtmSfc

    try:
      lineValuesLapseRate = lineValuesDeltaTemp/lineValuesDeltaHeightAtm #environmental lapse rate
    except ZeroDivisionError:
      lineValuesLapseRate = 6*lineValuesDeltaHeightAtm/1000 # Moist adiabatic lapse rate:  Varies, but about 6 deg C per km
    try:
      value = (lineValuesLapseRate*lineValuesDeltaHeightAtmSfc)+lineValuesRPT1000
      #print "    Value for current pixel (#", i + 1, "): ",value
    except ZeroDivisionError:
      value= Float.NaN
    except ValueError:
      value = Float.NaN
      #print "    Value for current pixel is cloud contaminated, value 'missing' assigned."
    if value < 223.15:
       value = Float.NaN #
    elif value > 313.15:
       value = Float.NaN
    else:
       value = value  
    lineValuesRTP950[0][i] = value
  newd = RTP950.clone()
  newd.setSamples(lineValuesRTP950)
  return newd

def tempModisRetAtmHeightN52ImageSeq(RTP950,RTP1000,RTA950,RTA1000,ASFC):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(RTP950)):
     newA = RTP950.clone()
     seq = RTP950.getDomainSet().getLength()
     for t in range(seq):
         print "    Computing ambient air temperature for image nr. ", t + 1, " of ", seq 
         seqRTP950 = RTP950.getSample(t)
         seqRTP1000 = RTP1000.getSample(t)
         seqRTA950 = RTA950.getSample(t)
         seqRTA1000 = RTA1000.getSample(t)
         newA.setSample(t,tempModisRetAtmHeightN52Image(seqRTP950,seqRTP1000,seqRTA950,seqRTA1000,ASFC))
     return newA
  else:
     return tempModisRetAtmHeightN52Image(RTP950,RTP1000,RTA950,RTA1000,ASFC)


##########################################################################################################################################################
# Relative humidity over water/ice using MODIS MOD07 dew point/ambient air temperature
##########################################################################################################################################################
# Purpose: 
# Creator: V. Venus
# 
sys.add_package('ucar.visad');
from ucar.visad import Util
from ucar.unidata.data.grid import GridUtil

def relHumModisAtmRetN52Image(Tair,Tdew):
  lineValuesTair  = Tair.getFloats()
  lineValuesTdew  = Tdew.getFloats()
  for i in xrange(Tair.getDomainSet().getLength()):
    pixelTair = lineValuesTair[0][i]
    pixelTdew = lineValuesTdew[0][i]
    try:
        # Source : Murphy and Koop, Review of the vapour pressure of ice and supercooled water for atmospheric applications, Q. J. R. Meteorol. Soc (2005), 131, pp. 1539-1565.
        if pixelTair > 273.15:
          # Calculate saturation pressure over liquid water
          # vapor pressure (e)
          es  = math.exp(54.842763 - 6763.22 / pixelTair - 4.210 * math.log(pixelTair) + 0.000367 * pixelTair + math.tanh(0.0415 * (pixelTair - 218.8)) * (53.878 - 1331.22 / pixelTair - 9.44523 * math.log(pixelTair) + 0.014025 * pixelTair)) / 100.
          # saturation vapor pressure (es)
          e = math.exp(54.842763 - 6763.22 / pixelTdew - 4.210 * math.log(pixelTdew) + 0.000367 * pixelTdew + math.tanh(0.0415 * (pixelTdew - 218.8)) * (53.878 - 1331.22 / pixelTdew - 9.44523 * math.log(pixelTdew) + 0.014025 * pixelTdew)) / 100.
          value = 100.*e/es # now calculate relative humidity...
          print "    Value for current pixel (#", i + 1, "): ", value
        else:
          # Calculate saturation pressure over liquid water
          # vapor pressure (e)
          es  = math.exp(9.550426 - 5723.265/ pixelTair + 3.53068 * math.log(pixelTair) - 0.00728332 * pixelTair) / 100.
          # saturation vapor pressure (es)
          e = math.exp(9.550426 - 5723.265/ pixelTdew + 3.53068 * math.log(pixelTdew) - 0.00728332 * pixelTdew) / 100.
          value = 100.*e/es # now calculate relative humidity...
          print "    Value for current pixel (#", i + 1, "): ", value
    except ZeroDivisionError:
      value= Float.NaN
      #print "    Value for current pixel cannot be calculated due to a devision by zero error, value '-999' assigned."
    except ValueError:
      value = Float.NaN
      print "    Input data for current pixel is missing, value 'NaN' aka 'not a number' assigned."
    lineValuesTair[0][i] = value
  newd = Tair.clone()
  newd.setSamples(lineValuesTair)
  #newunit = Util.parseUnit("percent")
  #newType = Util.makeRealType("relative humidity", newunit)
  #return GridUtil.setParamType(newd, newType)
  return newd

def relHumModisAtmRetN52ImageSeq(Tair,Tdew):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(Tair)):
     newA = Tair.clone()
     seq = Tair.getDomainSet().getLength()
     for t in range(seq):
         print "    Computing rel. humidity for image nr. ", t + 1, " of ", seq 
         seqTair = Tair.getSample(t)
         seqTdew = Tdew.getSample(t)
         newA.setSample(t,relHumModisAtmRetN52Image(seqTair,seqTdew))
     return newA
  else:
     return relHumModisAtmRetN52Image(Tair,Tdew)

##################################################################################################################################################################################################################################################
# Fire detection from FY-2C observations (four channel algorithm) with cloud Layer Mask
##############################################################################################################################################################################################################################################@####
# Purpose: Fire detection from FY-2C observations (four channel algorithm) with cloud Layer Mask
# Autrhor(s): V. Venus, Z.Xiao
# Usable displays: Images, Data Probe, 
# Used by: getFireDayNightImageSeqN52(T11,T12,T39,T87,GLC) for image sequences (see end of this section)
# Grouped under: User-satellite land surface obs
# see functional describtion of algorithm at:
# http://adde.itc.nl/IDV/operation/org/n52/operation.html
# version: 0.4
#   Channel     Wavelength (?m )
#   T11 :       10.3-11.3
#   T12 :       11.5-12.5
#   T7  :        6.3-7.6
#   T4  :        3.5-4.0

import sys;
sys.add_package('visad.meteorology');
sys.add_package('GridUtil');
from java.util import TimeZone
from visad import DateTime
from visad.meteorology import ImageSequence
from visad.meteorology import SingleBandedImage
sys.add_package('org.n52.math');
sys.add_package('org.n52.operation');
from org.n52.math import TerrainTriangulation
from org.n52.math import SatelliteTriangulation
from org.n52.math import SunTriangulation
from jarray import zeros
from java.lang import Float

def getFireDayNightImageN52(VIS,T4,T7,T11,T12,AVG_T4,AVG_T11):
  domain = T11.getDomainSet()
  cs = domain.getCoordinateSystem()
  len =  domain.getLength()
  #Note: here we want a copy of the range values so we don't pass in 0 (false) her
  lineValuesVIS = VIS.getFloats() #VIS
  lineValuesT11 = T11.getFloats() #T11
  lineValuesT12 = T12.getFloats() #T12
  lineValuesT4 = T4.getFloats() #T4
  lineValuesT7 = T7.getFloats() #T7
  lineValuesAVG_T4 = AVG_T4.getFloats() #  AVG_T4
  lineValuesAVG_T11 = AVG_T11.getFloats() #  AVG_T11
  
  #Pass in 0 as false so we don't copy the values
  samples = domain.getSamples(0)
  #Get the latlon points of the spatial domain
  latlon = cs.toReference(samples)
  #Clone the incoming data object
  newd = T11.clone()
  #Get time from image object
  tz = TimeZone.getTimeZone("GMT");
  if isinstance(T11, ImageSequence):
      sbi = T11.getImage(int(i))
      dateTime = sbi.getStartTime()
  elif isinstance(T11, SingleBandedImage):
      dateTime = T11.getStartTime()
  year = dateTime.formattedString("yyyy", tz)
  month = dateTime.formattedString("MM", tz)
  day = dateTime.formattedString("dd", tz)
  hour = float(dateTime.formattedString("H", tz))
  minutes = float(dateTime.formattedString("mm", tz))/60
  utc = hour + minutes
  #Intilialize the terrain triangulation
  tertr = TerrainTriangulation(6378136, 298.257)
  #Initialize the solar zenith angle (SZA) algorithm
  suntr = SunTriangulation(tertr, int(year), int(month), int(day), float(utc))
  solzenithalg = suntr.getZenithAngleAlgorithm()
  #Initialize sunrise and sunset algorithms
  sunrisealg = suntr.getSunRiseAlgorithm() 
  sunsetalg  = suntr.getSunSetAlgorithm()
  #Initialize the satellite zenith angle (VZA) algorithm
  sattr = SatelliteTriangulation(tertr, 0, 105, 5, 35784000)# satellite position, resolution[km] and altitude [km] for FY-2C
  satzenithalg = sattr.getZenithAngleAlgorithm()
  #Loop through image pixels and do the guts, but before that first declare the params object
  params = zeros(2,"d")
  for i in xrange(len):
    #fill the params object with latitude and longitude coordinates in decimal degrees 
    params[0] = latlon[0][i]
    params[1] = latlon[1][i]
   
    #calculate satellite angles
    #VZA = satzenithalg.calculate(params) #satellite zenith angle
    #VVA = math.asin(math.sin(VZA*math.pi/180)/6.61041658566) # The satellite-viewing angle (VVA) relates to VZA as follows:
                                                              # sinVZA = (satellite height + earth radius)/earth radius * sinVVA
                                                              #        = (   35784000 +   6378136   )/  6378136    * sinVVA
                                                              #        = 6.6104165856607635835924476994533 * sinVVA FOR FY-2C

    #first find out if the imagepixel was observed during night or day and run the appropriate algorithm
    sunrise = sunrisealg.calculate(params)
    sunset  = sunsetalg.calculate(params)

#############################################################
###########    Cloudy layer mask  ###########################
#############################################################
    #filter for cloud contaminated pixels and do the guts
    #if (lineValuesCLM[0][i] < 0):
        #CM = 0 #Cloud (Layer) Mask, where 1=cloudy and 0=clear sky
    #else:
        #CM = lineValuesCLM[0][i]
    # Compute the average of infrared chanels:
    Av = (lineValuesT4[0][i] + lineValuesT7[0][i] + lineValuesT11[0][i] + lineValuesT12[0][i])/4
   
    # Compute the standard deviation of infrared chanels:
    Sd = math.sqrt((math.pow((lineValuesT4[0][i]-Av),2)+ math.pow((lineValuesT7[0][i]-Av),2) + math.pow((lineValuesT11[0][i]-Av),2) + math.pow((lineValuesT12[0][i]-Av),2))/2)
    
    # Compute the standard deviation multiplied by the average of infrared chanels:
    AvSD = Av * Sd
    #Cloud mask (CM) # where 1=Cloudy and 0=clear sky   
    if (AvSD < 2890.7116): # or 2894.7116
        CM = 1
    else:
        CM = 0
##################################################################
##################################################################
    if (CM > 0):
        value = Float.NaN #cloud contaminated pixels so fill pixel value with "missing" or NaN         
    else: #clear-sky so calculate surface temperature
        if (utc < sunrise) and (utc > sunset): #it is night, so run the nighttime algorithm using the appropriate coefficients (i.e. "a(l)")
            #value =  a0 + a1*lineValuesT11[0][i] + a2*(lineValuesT11[0][i] - lineValuesT12[0][i]) + a3*(lineValuesT4[0][i] - lineValuesT7[0][i]) + a4*math.pow((lineValuesT11[0][i] - lineValuesT12[0][i]),2) + a5*((1/math.cos(VVA))-1)\
            df = lineValuesT4[0][i]-lineValuesT7[0][i]
            dp = lineValuesAVG_T4[0][i]-lineValuesAVG_T11[0][i]
            pf = df - dp 
            if (pf > 5) and (df > 30):
               value = 1
            else:
               value = 0 
        else: #it is day, so run the daytime algorithm using the appropriate coefficients (i.e. "a(l)"):
            #calculate solar zenith angle to reduce the solar signal contamination in the brightness temperature, which makes T4 decrease again
            #SZA = solzenithalg.calculate(params)
            #value =  a0 + a1*lineValuesT11[0][i] + a2*(lineValuesT11[0][i] - lineValuesT12[0][i]) + a3*(lineValuesT4[0][i] - lineValuesT7[0][i]) + a4*math.pow((lineValuesT11[0][i] - lineValuesT12[0][i]),2) + a5*((1/math.cos(VVA))-1) + a6*lineValuesT4[0][i]*math.cos(SZA)
            df = lineValuesT4[0][i]-lineValuesT7[0][i]
            dp = lineValuesAVG_T4[0][i]-lineValuesAVG_T11[0][i]
            refl = lineValuesVIS[0][i]  
            pf = df - dp 
            if (pf > 5) and (df > 30) and (refl <= 0.15):
               value = 1
            else:
               value = 0      
    lineValuesT11[0][i]  = value
  # set the new range values, and
  newd.setSamples(lineValuesT11)
  # Finish with returning the new data object
  return newd

# Time series functions of above (call this jython function for image sequences)
def getFireDayNightImageSeqN52(VIS,T4,T7,T11,T12,AVG_T4,AVG_T11):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(T11)):
     newA = T11.clone()
     seq = T11.getDomainSet().getLength()
     for t in range(seq):
         #print "    Computing LST for image nr. ", t + 1, " of ", seq 
         seqT11 = T11.getSample(t)
         seqT12 = T12.getSample(t)
         seqT4 = T4.getSample(t)
         seqT7 = T7.getSample(t)
         seqVIS = VIS.getSample(t)        
         newA.setSample(t,getFireDayNightImageN52(VIS,T4,T7,T11,T12,AVG_T4,AVG_T11))
     return newA
  else:
     return getFireDayNightImageN52(VIS,T4,T7,T11,T12,AVG_T4,AVG_T11)


##########################################################################################################################################################
# Brightness-to-Albedo conversion for FY-2C/MTSAT
##########################################################################################################################################################
#
# Purpose: Brightness-to-Albedo conversion for FY-2C/MTSAT
# Creator: V. Venus
# Usable in: Images, Data Probe
# Grouped under: 

from jarray import zeros

def getAlbedoVisFy2cMtsatAsImageN52(a):
  lineValuesA = a.getFloats()
  for i in xrange(a.getDomainSet().getLength()):
    Brightness = lineValuesA[0][i]
    value = math.pow((Brightness/25.5),2)
    lineValuesA[0][i] = value
  newd = a.clone()
  newd.setSamples(lineValuesA)
  #newunit = Util.parseUnit("percent")
  #newType = Util.makeRealType("Reflectivity", newunit)
  #return GridUtil.setParamType(newd, newType)
  return newd

def getAlbedoVisFy2cMtsatAsImageSeqN52(d):
  import ucar.unidata.data.grid.GridUtil as gu
  if (gu.isTimeSequence(d)):
     newA = d.clone()
     for t in range(d.getDomainSet().getLength()):
         subD = d.getSample(t)
         newA.setSample(t,getAlbedoVisFy2cMtsatAsImageN52(subD))
     return newA
  else:
     return getAlbedoVisFy2cMtsatAsImageN52(d)